"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.variantAlsoNegotiates = exports.uriTooLong = exports.upgradeRequired = exports.unsupportedMediaType = exports.unavailableForLegalReasons = exports.unauthorized = exports.tooManyRequests = exports.toResponse = exports.temporaryRedirect = exports.serviceUnavailable = exports.seeOther = exports.requestTimeout = exports.requestHeaderFieldsTooLarge = exports.rangeNotSatisfiable = exports.proxyAuthenticationRequired = exports.preconditionFailed = exports.permanentRedirect = exports.payloadTooLarge = exports.notModified = exports.notImplemented = exports.notFound = exports.notExtended = exports.notAcceptable = exports.networkAuthenticationRequired = exports.multipleChoices = exports.movedPermanently = exports.misdirectedRequest = exports.methodNotAllowed = exports.make = exports.lengthRequired = exports.isHttpError = exports.internalServerError = exports.imATeapot = exports.httpVersionNotSupported = exports.gone = exports.gatewayTimeout = exports.found = exports.forbidden = exports.expectationFailed = exports.conflict = exports.badRequest = exports.badGateway = exports.TypeId = void 0;
var internal = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("./internal/http-error.js"));
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/**
 * @since 1.0.0
 * @category type id
 */
const TypeId = exports.TypeId = internal.TypeId;
/**
 * @category destructors
 * @since 1.0.0
 */
const toResponse = exports.toResponse = internal.toResponse;
/**
 * @category refinements
 * @since 1.0.0
 */
const isHttpError = exports.isHttpError = internal.isHttpError;
/**
 * @category constructors
 * @since 1.0.0
 */
const make = exports.make = internal.make;
// *********************************************
// *                                           *
// *               4xx responses               *
// *                                           *
// *********************************************
/**
 * [400 Bad Request](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/400)
 *
 * The server cannot or will not process the request due to something that is perceived
 * to be a client error (e.g., malformed request syntax, invalid request message framing,
 * or deceptive request routing).
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const badRequest = exports.badRequest = /*#__PURE__*/internal.makeStatus(400);
/**
 * [401 Unauthorized](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/401)
 *
 * Although the HTTP standard specifies "unauthorized", semantically this response means
 * "unauthenticated". That is, the client must authenticate itself to get the requested response.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const unauthorized = exports.unauthorized = /*#__PURE__*/internal.makeStatus(401);
/**
 * [403 Forbidden](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/403)
 *
 * The client does not have access rights to the content; that is, it is unauthorized, so
 * the server is refusing to give the requested resource. Unlike 401 Unauthorized, the
 * client's identity is known to the server.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const forbidden = exports.forbidden = /*#__PURE__*/internal.makeStatus(403);
/**
 * [404 Not Found](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/404)
 *
 * The server cannot find the requested resource. In the browser, this means the URL is
 * not recognized. In an API, this can also mean that the endpoint is valid but the
 * resource itself does not exist. Servers may also send this response instead of 403
 * Forbidden to hide the existence of a resource from an unauthorized client. This
 * response code is probably the most well known due to its frequent occurrence on the web.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const notFound = exports.notFound = /*#__PURE__*/internal.makeStatus(404);
/**
 * [405 Method Not Allowed](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/405)
 *
 * The request method is known by the server but is not supported by the target resource.
 * For example, an API may not allow calling `DELETE` to remove a resource.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const methodNotAllowed = exports.methodNotAllowed = /*#__PURE__*/internal.makeStatus(405);
/**
 * [406 Not Acceptable](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/406)
 *
 * This response is sent when the web server, after performing [server-driven content
 * negotiation](https://developer.mozilla.org/en-US/docs/Web/HTTP/Content_negotiation#server-driven_content_negotiation),
 * doesn't find any content that conforms to the criteria given by the user agent.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const notAcceptable = exports.notAcceptable = /*#__PURE__*/internal.makeStatus(406);
/**
 * [407 Proxy Authentication Required](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/407)
 *
 * This is similar to 401 Unauthorized but authentication is needed to be done by a proxy.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const proxyAuthenticationRequired = exports.proxyAuthenticationRequired = /*#__PURE__*/internal.makeStatus(407);
/**
 * [408 Request Timeout](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/408)
 *
 * This response is sent on an idle connection by some servers, even without any previous
 * request by the client. It means that the server would like to shut down this unused
 * connection. This response is used much more since some browsers, like Chrome,
 * Firefox 27+, or IE9, use HTTP pre-connection mechanisms to speed up surfing. Also
 * note that some servers merely shut down the connection without sending this message.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const requestTimeout = exports.requestTimeout = /*#__PURE__*/internal.makeStatus(408);
/**
 * [409 Conflict](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/409)
 *
 * This response is sent when a request conflicts with the current state of the server.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const conflict = exports.conflict = /*#__PURE__*/internal.makeStatus(409);
/**
 * [410 Gone](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/410)
 *
 * This response is sent when the requested content has been permanently deleted from
 * server, with no forwarding address. Clients are expected to remove their caches
 * and links to the resource. The HTTP specification intends this status code to be
 * used for "limited-time, promotional services". APIs should not feel compelled to
 * indicate resources that have been deleted with this status code.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const gone = exports.gone = /*#__PURE__*/internal.makeStatus(410);
/**
 * [411 Length Required](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/411)
 *
 * Server rejected the request because the Content-Length header field is not defined
 * and the server requires it.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const lengthRequired = exports.lengthRequired = /*#__PURE__*/internal.makeStatus(411);
/**
 * [412 Precondition Failed](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/412)
 *
 * The client has indicated preconditions in its headers which the server does not meet.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const preconditionFailed = exports.preconditionFailed = /*#__PURE__*/internal.makeStatus(412);
/**
 * [413 Payload Too Large](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/413)
 *
 * Request entity is larger than limits defined by server. The server might close the
 * connection or return an Retry-After header field.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const payloadTooLarge = exports.payloadTooLarge = /*#__PURE__*/internal.makeStatus(413);
/**
 * [414 URI Too Long](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/414)
 *
 * The URI requested by the client is longer than the server is willing to interpret.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const uriTooLong = exports.uriTooLong = /*#__PURE__*/internal.makeStatus(414);
/**
 * [415 Unsupported Media Type](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/415)
 *
 * The media format of the requested data is not supported by the server, so the
 * server is rejecting the request.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const unsupportedMediaType = exports.unsupportedMediaType = /*#__PURE__*/internal.makeStatus(415);
/**
 * [416 Range Not Satisfiable](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/416)
 *
 * The range specified by the Range header field in the request cannot be fulfilled. It's
 * possible that the range is outside the size of the target URI's data.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const rangeNotSatisfiable = exports.rangeNotSatisfiable = /*#__PURE__*/internal.makeStatus(416);
/**
 * [417 Expectation Failed](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/417)
 *
 * This response code means the expectation indicated by the Expect request header field
 * cannot be met by the server.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const expectationFailed = exports.expectationFailed = /*#__PURE__*/internal.makeStatus(417);
/**
 * [418 I'm a teapot](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/418)
 *
 * The server refuses the attempt to brew coffee with a teapot.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const imATeapot = exports.imATeapot = /*#__PURE__*/internal.makeStatus(418);
/**
 * [421 Misdirected Request](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/421)
 *
 * The request was directed at a server that is not able to produce a response. This
 * can be sent by a server that is not configured to produce responses for the
 * combination of scheme and authority that are included in the request URI.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const misdirectedRequest = exports.misdirectedRequest = /*#__PURE__*/internal.makeStatus(421);
/**
 * [426 Upgrade Required](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/426)
 *
 * The server refuses to perform the request using the current protocol but might be
 * willing to do so after the client upgrades to a different protocol. The server
 * sends an Upgrade header in a 426 response to indicate the required protocol(s).
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const upgradeRequired = exports.upgradeRequired = /*#__PURE__*/internal.makeStatus(426);
/**
 * [429 Too Many Requests](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/429)
 *
 * The user has sent too many requests in a given amount of time ("rate limiting").
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const tooManyRequests = exports.tooManyRequests = /*#__PURE__*/internal.makeStatus(429);
/**
 * [431 Request Header Fields Too Large](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/431)
 *
 * The server is unwilling to process the request because its header fields are too large.
 * The request may be resubmitted after reducing the size of the request header fields.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const requestHeaderFieldsTooLarge = exports.requestHeaderFieldsTooLarge = /*#__PURE__*/internal.makeStatus(431);
/**
 * [451 Unavailable For Legal Reasons](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/451)
 *
 * The user agent requested a resource that cannot legally be provided, such as a web
 * page censored by a government.
 *
 * @category 4xx constructors
 * @since 1.0.0
 */
const unavailableForLegalReasons = exports.unavailableForLegalReasons = /*#__PURE__*/internal.makeStatus(451);
// *********************************************
// *                                           *
// *               5xx responses               *
// *                                           *
// *********************************************
/**
 * [500 Internal Server Error](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/500)
 *
 * The server has encountered a situation it does not know how to handle.
 *
 * @category 5xx constructors
 * @since 1.0.0
 */
const internalServerError = exports.internalServerError = /*#__PURE__*/internal.makeStatus(500);
/**
 * [501 Not Implemented](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/501)
 *
 * The request method is not supported by the server and cannot be handled. The only
 * methods that servers are required to support (and therefore that must not return
 * this code) are GET and HEAD.
 *
 * @category 5xx constructors
 * @since 1.0.0
 */
const notImplemented = exports.notImplemented = /*#__PURE__*/internal.makeStatus(501);
/**
 * [502 Bad Gateway](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/502)
 *
 * This error response means that the server, while working as a gateway to get a
 * response needed to handle the request, got an invalid response.
 *
 * @category 5xx constructors
 * @since 1.0.0
 */
const badGateway = exports.badGateway = /*#__PURE__*/internal.makeStatus(502);
/**
 * [503 Service Unavailable](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/503)
 *
 * The server is not ready to handle the request. Common causes are a server that
 * is down for maintenance or that is overloaded. Note that together with this response,
 * a user-friendly page explaining the problem should be sent. This response should
 * be used for temporary conditions and the Retry-After HTTP header should, if possible,
 * contain the estimated time before the recovery of the service. The webmaster must
 * also take care about the caching-related headers that are sent along with this
 * response, as these temporary condition responses should usually not be cached.
 *
 * @category 5xx constructors
 * @since 1.0.0
 */
const serviceUnavailable = exports.serviceUnavailable = /*#__PURE__*/internal.makeStatus(503);
/**
 * [504 Gateway Timeout](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/504)
 *
 * This error response is given when the server is acting as a gateway and
 * cannot get a response in time.
 *
 * @category 5xx constructors
 * @since 1.0.0
 */
const gatewayTimeout = exports.gatewayTimeout = /*#__PURE__*/internal.makeStatus(504);
/**
 * [505 HTTP Version Not Supported](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/505)
 *
 * The HTTP version used in the request is not supported by the server.
 *
 * @category 5xx constructors
 * @since 1.0.0
 */
const httpVersionNotSupported = exports.httpVersionNotSupported = /*#__PURE__*/internal.makeStatus(505);
/**
 * [506 Variant Also Negotiates](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/506)
 *
 * The server has an internal configuration error: the chosen variant resource is
 * configured to engage in transparent content negotiation itself, and is
 * therefore not a proper end point in the negotiation process.
 *
 * @category 5xx constructors
 * @since 1.0.0
 */
const variantAlsoNegotiates = exports.variantAlsoNegotiates = /*#__PURE__*/internal.makeStatus(506);
/**
 * [510 Not Extended](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/510)
 *
 * Further extensions to the request are required for the server to fulfill it.
 *
 * @category 5xx constructors
 * @since 1.0.0
 */
const notExtended = exports.notExtended = /*#__PURE__*/internal.makeStatus(510);
/**
 * [511 Network Authentication Required](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/511)
 *
 * Indicates that the client needs to authenticate to gain network access.
 *
 * @category 5xx constructors
 * @since 1.0.0
 */
const networkAuthenticationRequired = exports.networkAuthenticationRequired = /*#__PURE__*/internal.makeStatus(511);
// *********************************************
// *                                           *
// *               3xx responses               *
// *                                           *
// *********************************************
/**
 * [300 Multiple Choices](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/300)
 *
 * The request has more than one possible response. The user agent or user should choose one of them.
 * (There is no standardized way of choosing one of the responses, but HTML links to the possibilities
 * are recommended so the user can pick.)
 *
 * @category 3xx constructors
 * @since 1.0.0
 */
const multipleChoices = exports.multipleChoices = /*#__PURE__*/internal.makeStatus(300);
/**
 * [301 Moved Permanently](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/301)
 *
 * The URL of the requested resource has been changed permanently. The new URL is given in the response.
 *
 * @category 3xx constructors
 * @since 1.0.0
 */
const movedPermanently = exports.movedPermanently = /*#__PURE__*/internal.makeStatus(301);
/**
 * [302 Found](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/302)
 *
 * This response code means that the URI of requested resource has been changed temporarily. Further
 * changes in the URI might be made in the future. Therefore, this same URI should be used by the
 * client in future requests.
 *
 * @category 3xx constructors
 * @since 1.0.0
 */
const found = exports.found = /*#__PURE__*/internal.makeStatus(302);
/**
 * [303 See Other](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/303)
 *
 * The server sent this response to direct the client to get the requested resource at another URI
 * with a GET request.
 *
 * @category 3xx constructors
 * @since 1.0.0
 */
const seeOther = exports.seeOther = /*#__PURE__*/internal.makeStatus(303);
/**
 * [304 Not Modified](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/304)
 *
 * This is used for caching purposes. It tells the client that the response has not been modified,
 * so the client can continue to use the same cached version of the response.
 *
 * @category 3xx constructors
 * @since 1.0.0
 */
const notModified = exports.notModified = /*#__PURE__*/internal.makeStatus(304);
/**
 * [307 Temporary Redirect](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/307)
 *
 * The server sends this response to direct the client to get the requested resource at another URI
 * with the same method that was used in the prior request. This has the same semantics as the
 * `302 Found` HTTP response code, with the exception that the user agent must not change the HTTP
 * method * used: if a `POST` was used in the first request, a `POST` must be used in the second request.
 *
 * @category 3xx constructors
 * @since 1.0.0
 */
const temporaryRedirect = exports.temporaryRedirect = /*#__PURE__*/internal.makeStatus(307);
/**
 * [308 Permanent Redirect](https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/308)
 *
 * This means that the resource is now permanently located at another URI, specified by the
 * `Location:` HTTP Response header. This has the same semantics as the 301 Moved Permanently HTTP
 * response code, with the exception that the user agent must not change the HTTP method used:
 * if a POST was used in the first request, a POST must be used in the second request.
 *
 * @category 3xx constructors
 * @since 1.0.0
 */
const permanentRedirect = exports.permanentRedirect = /*#__PURE__*/internal.makeStatus(308);
//# sourceMappingURL=HttpError.js.map