"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.variance = exports.validateEndpoint = exports.setSecurity = exports.setResponseStatus = exports.setResponseRepresentations = exports.setResponseHeaders = exports.setResponseBody = exports.setResponse = exports.setRequestQuery = exports.setRequestPath = exports.setRequestHeaders = exports.setRequestBody = exports.setRequest = exports.setOptions = exports.make = exports.isFullResponse = exports.isApiEndpoint = exports.getSecurity = exports.getResponse = exports.getRequest = exports.getPath = exports.getOptions = exports.getMethod = exports.getId = exports.addResponse = exports.TypeId = void 0;
var Array = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Array"));
var Equivalence = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Equivalence"));
var _Function = /*#__PURE__*/require("effect/Function");
var Order = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Order"));
var Pipeable = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Pipeable"));
var Predicate = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Predicate"));
var Tuple = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Tuple"));
var ApiRequest = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiRequest.js"));
var ApiResponse = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiResponse.js"));
var ApiSchema = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiSchema.js"));
var Security = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../Security.js"));
var api_request = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("./api-request.js"));
var api_response = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("./api-response.js"));
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/** @internal */
const TypeId = exports.TypeId = /*#__PURE__*/Symbol.for("effect-http/Api/EndpointTypeId");
/** @internal */
const variance = exports.variance = {
  /* c8 ignore next */
  _Id: _ => _,
  /* c8 ignore next */
  _Request: _ => _,
  /* c8 ignore next */
  _Response: _ => _,
  /* c8 ignore next */
  _Security: _ => _
};
/** @internal */
class ApiEndpointImpl {
  id;
  path;
  method;
  request;
  responses;
  security;
  options;
  [TypeId] = variance;
  constructor(id, path, method, request, responses, security, options) {
    this.id = id;
    this.path = path;
    this.method = method;
    this.request = request;
    this.responses = responses;
    this.security = security;
    this.options = options;
  }
  pipe() {
    return Pipeable.pipeArguments(this, arguments);
  }
}
/** @internal */
const make = (method, id, path, options) => new ApiEndpointImpl(id, path, method, api_request.defaultRequest, [api_response.defaultResponse], Security.unit, {
  ...options
});
/** @internal */
exports.make = make;
const setRequest = request => endpoint => new ApiEndpointImpl(getId(endpoint), getPath(endpoint), getMethod(endpoint), request, getResponse(endpoint), getSecurity(endpoint), getOptions(endpoint));
/** @internal */
exports.setRequest = setRequest;
const setRequestBody = schema => endpoint => {
  if (getMethod(endpoint) === "GET") {
    throw new Error(`GET ${getPath(endpoint)} (${getId(endpoint)}) cannot have a request body`);
  }
  return setRequest(ApiRequest.setBody(schema)(getRequest(endpoint)))(endpoint);
};
/** @internal */
exports.setRequestBody = setRequestBody;
const setRequestPath = schema => endpoint => setRequest(ApiRequest.setPath(schema)(getRequest(endpoint)))(endpoint);
/** @internal */
exports.setRequestPath = setRequestPath;
const setRequestQuery = schema => endpoint => setRequest(ApiRequest.setQuery(schema)(getRequest(endpoint)))(endpoint);
/** @internal */
exports.setRequestQuery = setRequestQuery;
const setRequestHeaders = schema => endpoint => setRequest(ApiRequest.setHeaders(schema)(getRequest(endpoint)))(endpoint);
/** @internal */
exports.setRequestHeaders = setRequestHeaders;
const setSecurity = security => endpoint => new ApiEndpointImpl(getId(endpoint), getPath(endpoint), getMethod(endpoint), getRequest(endpoint), getResponse(endpoint), security, getOptions(endpoint));
/** @internal */
exports.setSecurity = setSecurity;
const setOptions = options => endpoint => new ApiEndpointImpl(getId(endpoint), getPath(endpoint), getMethod(endpoint), getRequest(endpoint), getResponse(endpoint), getSecurity(endpoint), {
  ...getOptions(endpoint),
  ...options
});
/** @internal */
exports.setOptions = setOptions;
const getId = endpoint => endpoint.id;
/** @internal */
exports.getId = getId;
const getRequest = endpoint => endpoint.request;
/** @internal */
exports.getRequest = getRequest;
const getResponse = endpoint => endpoint.responses;
/** @internal */
exports.getResponse = getResponse;
const getSecurity = endpoint => endpoint.security;
/** @internal */
exports.getSecurity = getSecurity;
const getMethod = endpoint => endpoint.method;
/** @internal */
exports.getMethod = getMethod;
const getPath = endpoint => endpoint.path;
/** @internal */
exports.getPath = getPath;
const getOptions = endpoint => endpoint.options;
/** @internal */
exports.getOptions = getOptions;
const setResponse = response => endpoint => {
  const newResponse = ApiResponse.isApiResponse(response) ? response : ApiResponse.make(response.status, response.body, response.headers);
  return new ApiEndpointImpl(getId(endpoint), getPath(endpoint), getMethod(endpoint), getRequest(endpoint), [newResponse], getSecurity(endpoint), getOptions(endpoint));
};
/** @internal */
exports.setResponse = setResponse;
const setResponseStatus = status => endpoint => {
  const responses = getResponse(endpoint);
  if (responses.length !== 1) {
    throw new Error("Expected exactly one response");
  }
  return setResponse(ApiResponse.setStatus(status)(responses[0]))(endpoint);
};
/** @internal */
exports.setResponseStatus = setResponseStatus;
const setResponseBody = schema => endpoint => {
  const responses = getResponse(endpoint);
  if (responses.length !== 1) {
    throw new Error("Expected exactly one response");
  }
  return setResponse(ApiResponse.setBody(schema)(responses[0]))(endpoint);
};
/** @internal */
exports.setResponseBody = setResponseBody;
const setResponseHeaders = schema => endpoint => {
  const responses = getResponse(endpoint);
  if (responses.length !== 1) {
    throw new Error("Expected exactly one response");
  }
  return setResponse(ApiResponse.setHeaders(schema)(responses[0]))(endpoint);
};
exports.setResponseHeaders = setResponseHeaders;
const addResponse = response => endpoint => {
  const responses = getResponse(endpoint);
  const newResponse = ApiResponse.isApiResponse(response) ? response : ApiResponse.make(response.status, response.body, response.headers);
  if (responses.some(r => ApiResponse.getStatus(r) === ApiResponse.getStatus(newResponse))) {
    throw new Error(`Status code ${ApiResponse.getStatus(newResponse)} already exists`);
  }
  return new ApiEndpointImpl(getId(endpoint), getPath(endpoint), getMethod(endpoint), getRequest(endpoint), [...responses, newResponse], getSecurity(endpoint), getOptions(endpoint));
};
exports.addResponse = addResponse;
const setResponseRepresentations = representations => endpoint => {
  const responses = getResponse(endpoint);
  if (responses.length !== 1) {
    throw new Error("Expected exactly one response");
  }
  return new ApiEndpointImpl(getId(endpoint), getPath(endpoint), getMethod(endpoint), getRequest(endpoint), [ApiResponse.setRepresentations(representations)(responses[0])], getSecurity(endpoint), getOptions(endpoint));
};
exports.setResponseRepresentations = setResponseRepresentations;
const isFullResponse = endpoint => {
  const response = getResponse(endpoint);
  const successResponses = response.filter(response => ApiResponse.getStatus(response) < 300);
  const hasMultipleSuccessResponses = successResponses.length > 1;
  const hasHeaders = !ApiSchema.isIgnored(ApiResponse.getHeadersSchema(response[0]));
  return hasMultipleSuccessResponses || hasHeaders;
};
/** @internal */
exports.isFullResponse = isFullResponse;
const getSchemaPropertySignatures = schema => {
  let ast = schema.ast;
  if (ast._tag === "Transformation") {
    ast = ast.from;
  }
  if (ast._tag !== "TypeLiteral") {
    throw new Error(`Response headers must be a type literal schema`);
  }
  return ast.propertySignatures;
};
/** @internal */
const tupleOrder = /*#__PURE__*/Order.tuple(Order.string, Order.boolean);
/** @internal */
const tupleEquivalence = /*#__PURE__*/Equivalence.tuple(Equivalence.string, Equivalence.boolean);
/** @internal */
const arrayOfTupleEquals = /*#__PURE__*/Array.getEquivalence(tupleEquivalence);
/** @internal */
const checkPathPatternMatchesSchema = (id, path, schema) => {
  const fromSchema = (0, _Function.pipe)(ApiSchema.isIgnored(schema) ? [] : getSchemaPropertySignatures(schema), Array.fromIterable, Array.map(ps => [ps.name, ps.isOptional]), Array.sort(tupleOrder));
  const fromPath = (0, _Function.pipe)(path.matchAll(/:(\w+)[?]?/g), Array.fromIterable, Array.map(([name]) => {
    if (name.endsWith("?")) {
      return [name.slice(1, -1), true];
    }
    return [name.slice(1), false];
  }), Array.sort(tupleOrder));
  const matched = arrayOfTupleEquals(fromPath, fromSchema);
  if (!matched) {
    const pathNames = fromPath.map(Tuple.getFirst).join(", ");
    const schemaNames = fromSchema.map(Tuple.getFirst).join(", ");
    throw new Error(`Path doesn't match the path schema (endpoint: "${id}").\nfrom path - ${pathNames}\nfrom schema - ${schemaNames}`);
  }
};
/** @internal */
const validateEndpoint = endpoint => {
  checkPathPatternMatchesSchema(getId(endpoint), getPath(endpoint), ApiRequest.getPathSchema(getRequest(endpoint)));
};
/** @internal */
exports.validateEndpoint = validateEndpoint;
const isApiEndpoint = u => Predicate.hasProperty(u, TypeId) && Predicate.isObject(u[TypeId]);
exports.isApiEndpoint = isApiEndpoint;
//# sourceMappingURL=api-endpoint.js.map