"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setBearer = exports.setBasicAuth = exports.setApiKey = exports.make = exports.endpointClient = void 0;
var FetchHttpClient = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("@effect/platform/FetchHttpClient"));
var HttpClient = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("@effect/platform/HttpClient"));
var HttpClientRequest = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("@effect/platform/HttpClientRequest"));
var Context = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Context"));
var Effect = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Effect"));
var Encoding = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Encoding"));
var _Function = /*#__PURE__*/require("effect/Function");
var Layer = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Layer"));
var Api = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../Api.js"));
var ApiEndpoint = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiEndpoint.js"));
var ClientError = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ClientError.js"));
var ClientRequestEncoder = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("./clientRequestEncoder.js"));
var ClientResponseParser = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("./clientResponseParser.js"));
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/** @internal */
const defaultHttpClient = /*#__PURE__*/FetchHttpClient.layer.pipe(Layer.build, /*#__PURE__*/Effect.map(/*#__PURE__*/Context.get(HttpClient.HttpClient)), Effect.scoped, Effect.runSync);
/** @internal */
const endpointClient = (id, api, options) => {
  const endpoint = Api.getEndpoint(api, id);
  const responseParser = ClientResponseParser.create(endpoint);
  const requestEncoder = ClientRequestEncoder.create(endpoint);
  const httpClient = (options.httpClient ?? defaultHttpClient).pipe(options.baseUrl ? HttpClient.mapRequest(HttpClientRequest.prependUrl(options.baseUrl)) : _Function.identity);
  return (args, mapper) => (0, _Function.pipe)(requestEncoder.encodeRequest(args), Effect.map(mapper ?? _Function.identity), Effect.flatMap(httpClient.execute), Effect.catchTags({
    RequestError: err => ClientError.makeClientSide(err.cause, err.message),
    ResponseError: err => ClientError.makeServerSide(err.cause, err.response.status, err.message),
    ClientError: err => Effect.fail(err)
  }), Effect.flatMap(responseParser.parseResponse), Effect.annotateLogs("clientOperationId", ApiEndpoint.getId(endpoint)));
};
/** @internal */
exports.endpointClient = endpointClient;
const make = (api, options) => api.groups.flatMap(group => group.endpoints).reduce((client, endpoint) => ({
  ...client,
  [ApiEndpoint.getId(endpoint)]: endpointClient(ApiEndpoint.getId(endpoint), api, options ?? {})
}), {});
/** @internal */
exports.make = make;
const setBasicAuth = exports.setBasicAuth = /*#__PURE__*/(0, _Function.dual)(3, (request, user, pass) => HttpClientRequest.setHeader(request, "Authorization", `Basic ${Encoding.encodeBase64(`${user}:${pass}`)}`));
/** @internal */
const setBearer = exports.setBearer = /*#__PURE__*/(0, _Function.dual)(2, (request, token) => HttpClientRequest.setHeader(request, "Authorization", `Bearer ${token}`));
/** @internal */
const setApiKey = exports.setApiKey = /*#__PURE__*/(0, _Function.dual)(4, (request, key, _in, apiKey) => _in === "query" ? HttpClientRequest.setUrlParam(request, key, apiKey) : HttpClientRequest.setHeader(request, key, apiKey));
//# sourceMappingURL=client.js.map