"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.create = void 0;
var HttpClientRequest = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("@effect/platform/HttpClientRequest"));
var Effect = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Effect"));
var _Function = /*#__PURE__*/require("effect/Function");
var Predicate = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Predicate"));
var Schema = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Schema"));
var ApiEndpoint = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiEndpoint.js"));
var ApiRequest = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiRequest.js"));
var ApiSchema = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiSchema.js"));
var ClientError = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ClientError.js"));
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/** @internal */
const make = encodeRequest => ({
  encodeRequest
});
/** @internal */
const create = endpoint => {
  const encodeBody = createBodyEncoder(endpoint);
  const encodeQuery = createQueryEncoder(endpoint);
  const encodeHeaders = createHeadersEncoder(endpoint);
  const encodePath = createPathEncoder(endpoint);
  return make(input => Effect.gen(function* (_) {
    const _input = input || {};
    const body = yield* _(encodeBody(_input["body"]));
    const query = yield* _(encodeQuery(_input["query"]));
    const path = yield* _(encodePath(_input["path"]));
    const headers = yield* _(encodeHeaders(_input["headers"]));
    const finalPath = constructPath(path || {}, ApiEndpoint.getPath(endpoint));
    const request = (0, _Function.pipe)(HttpClientRequest.get(finalPath), HttpClientRequest.setMethod(ApiEndpoint.getMethod(endpoint)), body === undefined ? _Function.identity : body instanceof FormData ? HttpClientRequest.bodyFormData(body) : HttpClientRequest.bodyUnsafeJson(body), query ? HttpClientRequest.setUrlParams(query) : _Function.identity, headers ? HttpClientRequest.setHeaders(headers) : _Function.identity);
    return request;
  }));
};
/** @internal */
exports.create = create;
const ignoredSchemaEncoder = name => input => {
  if (input !== undefined) {
    return Effect.dieMessage(`Unexpected ${name} provided, got ${JSON.stringify(input)}`);
  }
  return Effect.succeed(undefined);
};
/** @internal */
const createBodyEncoder = endpoint => {
  const schema = ApiRequest.getBodySchema(ApiEndpoint.getRequest(endpoint));
  if (ApiSchema.isIgnored(schema)) {
    return ignoredSchemaEncoder("body");
  }
  const encode = Schema.encode(schema);
  return body => {
    return encode(body).pipe(Effect.mapError(error => ClientError.makeClientSide(error, `Failed to encode body. ${error.message}`)));
  };
};
/** @internal */
const isRecordOrUndefined = i => Predicate.isRecord(i) || Predicate.isUndefined(i);
/** @internal */
const createQueryEncoder = endpoint => {
  const schema = ApiRequest.getQuerySchema(ApiEndpoint.getRequest(endpoint));
  if (ApiSchema.isIgnored(schema)) {
    return ignoredSchemaEncoder("query");
  }
  const encode = Schema.encode(schema);
  return query => {
    return encode(query).pipe(Effect.mapError(error => ClientError.makeClientSide(error, `Failed to encode query parameters. ${error.message}`)));
  };
};
/** @internal */
const createHeadersEncoder = endpoint => {
  const schema = ApiRequest.getHeadersSchema(ApiEndpoint.getRequest(endpoint));
  const encode = ApiSchema.isIgnored(schema) ? undefined : Schema.encode(schema);
  return headers => {
    if (!isRecordOrUndefined(headers)) {
      return Effect.dieMessage("Headers must be a record");
    }
    return (encode ?? Effect.succeed)(headers).pipe(Effect.mapError(error => ClientError.makeClientSide(error, `Failed to encode headers. ${error.message}`)));
  };
};
/** @internal */
const createPathEncoder = endpoint => {
  const schema = ApiRequest.getPathSchema(ApiEndpoint.getRequest(endpoint));
  if (ApiSchema.isIgnored(schema)) {
    return ignoredSchemaEncoder("path");
  }
  const encode = Schema.encode(schema);
  return params => {
    return encode(params).pipe(Effect.mapError(error => ClientError.makeClientSide(error, `Failed to encode path parmeters. ${error.message}.`)));
  };
};
/** @internal */
const constructPath = (params, path) => {
  return Object.entries(params ?? {}).reduce((path, [key, value]) => path.replace(new RegExp(`(:${key})(\\?)?`), value), path).replace(/\/:(\w+)(\?)?/, "");
};
//# sourceMappingURL=clientRequestEncoder.js.map