"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.create = void 0;
var Array = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Array"));
var Effect = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Effect"));
var _Function = /*#__PURE__*/require("effect/Function");
var Option = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Option"));
var Schema = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Schema"));
var Unify = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Unify"));
var ApiEndpoint = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiEndpoint.js"));
var ApiResponse = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiResponse.js"));
var ApiSchema = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiSchema.js"));
var ClientError = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ClientError.js"));
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/** @internal */
const make = parseResponse => ({
  parseResponse
});
/** @internal */
const create = endpoint => {
  const responses = ApiEndpoint.getResponse(endpoint);
  const isFullResponse = ApiEndpoint.isFullResponse(endpoint);
  const statusToSchema = responses.reduce((obj, schemas) => ({
    ...obj,
    [ApiResponse.getStatus(schemas)]: schemas
  }), {});
  return make(response => Effect.gen(function* (_) {
    yield* _(handleUnsucessful(response));
    if (!(response.status in statusToSchema)) {
      const allowedStatuses = Object.keys(statusToSchema);
      return yield* _(ClientError.makeClientSide(`Unexpected status ${response.status}. Allowed ones are ${allowedStatuses}.`));
    }
    const responseSpec = statusToSchema[response.status];
    const _parseBody = parseBody(ApiResponse.getBodySchema(responseSpec), ApiResponse.getRepresentations(responseSpec));
    const body = yield* _(_parseBody(response));
    if (!isFullResponse) {
      return body;
    }
    const headersSchema = ApiResponse.getHeadersSchema(responseSpec);
    const headers = ApiSchema.isIgnored(headersSchema) ? undefined : yield* _(response.headers, Schema.decodeUnknown(headersSchema), Effect.mapError(ClientError.makeClientSideResponseValidation("headers")));
    return {
      status: response.status,
      body,
      headers
    };
  }));
};
/** @internal */
exports.create = create;
const handleUnsucessful = /*#__PURE__*/Unify.unify(response => {
  if (response.status >= 300) {
    return response.json.pipe(Effect.orElse(() => response.text), Effect.orElseSucceed(() => "No body provided"), Effect.flatMap(error => Effect.fail(ClientError.makeServerSide(error, response.status))));
  }
  return Effect.void;
});
/** @internal */
const representationFromResponse = (representations, response) => {
  if (representations.length === 0) {
    return representations[0];
  }
  const contentType = response.headers["content-type"];
  // TODO: this logic needs to be improved a lot!
  return (0, _Function.pipe)(representations, Array.filter(representation => representation.contentType === contentType), Array.head, Option.getOrElse(() => representations[0]));
};
/** @internal */
const decodeBody = (schema, representations) => {
  const parse = Schema.decodeUnknown(schema);
  return response => {
    const representation = representationFromResponse(representations, response);
    return response.text.pipe(Effect.mapError(error => ClientError.makeClientSide(error, `Invalid response: ${error.reason}`)), Effect.flatMap((0, _Function.flow)(representation.parse, Effect.mapError(error => ClientError.makeClientSide(error, `Invalid response: ${error.message}`)))), Effect.flatMap((0, _Function.flow)(parse, Effect.mapError(ClientError.makeClientSideResponseValidation("body")))));
  };
};
/** @internal */
const parseBody = (schema, representations) => {
  if (ApiSchema.isIgnored(schema)) {
    return () => Effect.succeed(undefined);
  }
  return decodeBody(schema, representations);
};
//# sourceMappingURL=clientResponseParser.js.map