"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.randomExample = void 0;
var Array = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Array"));
var Context = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Context"));
var Data = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Data"));
var Effect = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Effect"));
var _Function = /*#__PURE__*/require("effect/Function");
var Option = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Option"));
var Ref = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Ref"));
var Schema = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Schema"));
var AST = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/SchemaAST"));
var Unify = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Unify"));
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/** @internal */
const getExampleValue = ast => (0, _Function.pipe)(AST.getAnnotation(AST.ExamplesAnnotationId)(ast), Option.getOrUndefined);
/** @internal */
const getDescription = ast => (0, _Function.pipe)(AST.getAnnotation(AST.DescriptionAnnotationId)(ast), Option.getOrUndefined);
/** @internal */
const IntegerCounter = /*#__PURE__*/Context.GenericTag("effect-http/example_compiler/integer-counter");
/** @internal */
const nextInteger = /*#__PURE__*/Effect.flatMap(IntegerCounter, /*#__PURE__*/Ref.getAndUpdate(n => n + 1));
/** @internal */
const randomChoice = xs => (0, _Function.pipe)(Effect.randomWith(r => r.nextIntBetween(0, xs.length)), Effect.filterOrFail(i => i >= 0, () => RandomExampleErrorImpl.make(`Can choose from ${xs}`)), Effect.map(i => xs[i]));
/** @internal */
class RandomExampleErrorImpl extends /*#__PURE__*/Data.TaggedError("RandomExampleError") {
  static make(error) {
    return new RandomExampleErrorImpl({
      error
    });
  }
}
/** @internal */
const randomExample = schema => {
  const go = (ast, constraint) => {
    const exampleFromAnnotation = getExampleValue(ast);
    if (exampleFromAnnotation) {
      return randomChoice(exampleFromAnnotation);
    }
    switch (ast._tag) {
      case "Literal":
        {
          return Effect.succeed(ast.literal);
        }
      case "UnknownKeyword":
        return Effect.succeed(undefined);
      case "AnyKeyword":
        return randomChoice([{}, "hello-world", 69, undefined]);
      case "StringKeyword":
        return randomChoice(["hello world", "patrik"]);
      case "NumberKeyword":
        return Effect.map(nextInteger, number => {
          if (constraint) {
            return resolveConstrainedNumber(number, constraint);
          }
          return number;
        });
      case "BooleanKeyword":
        return randomChoice([true, false]);
      case "ObjectKeyword":
        return randomChoice([{
          iam: "object"
        }]);
      case "TupleType":
        {
          const generate = Effect.forEach(element => go(element, undefined));
          const elements = generate(ast.elements.map(element => element.type));
          const postRestElementsAst = (0, _Function.pipe)(Array.tail(ast.rest), Option.map(rest => rest.map(e => e.type)), Option.getOrElse(() => []));
          const nElements = ast.elements.length + postRestElementsAst.length;
          const rest = (0, _Function.pipe)(Array.head(ast.rest), Option.map(rest => globalThis.Array(constraint?.min != null ? constraint.min - nElements : 1).fill(rest.type)), Option.map(generate), Option.getOrElse(() => Effect.succeed([])));
          const postRestElements = generate(postRestElementsAst);
          return Effect.mergeAll([elements, rest, postRestElements], [], (a, b) => [...a, ...b]);
        }
      case "TypeLiteral":
        {
          if (ast.indexSignatures.length < ast.indexSignatures.filter(is => is.parameter._tag === "StringKeyword").length) {
            return Effect.fail(RandomExampleErrorImpl.make(`Cannot create example for some index signatures`));
          }
          const result = (0, _Function.pipe)(ast.propertySignatures, Effect.reduce({}, (acc, ps) => Effect.map(go(ps.type, constraint), v => ({
            ...acc,
            [ps.name]: v
          }))));
          return result;
        }
      case "Union":
        {
          return Effect.flatten(randomChoice(ast.types.map(ast => go(ast, constraint))));
        }
      case "Enums":
        {
          return randomChoice(ast.enums);
        }
      case "Refinement":
        {
          return createConstraintFromRefinement(ast).pipe(Effect.flatMap(constraint2 => go(ast.from, {
            ...constraint,
            ...constraint2
          })), Effect.flatMap(a => {
            const error = ast.filter(a, AST.defaultParseOption, ast);
            if (Option.isNone(error)) {
              return Effect.succeed(a);
            }
            return Effect.fail(RandomExampleErrorImpl.make(`Cannot create an example for refinement ${JSON.stringify(error, (_, value) => typeof value === "bigint" ? value.toString() : value)}`));
          }));
        }
      case "Transformation":
        return go(ast.to, constraint);
      case "UniqueSymbol":
        return Effect.fail(RandomExampleErrorImpl.make(`UniqueSymbol`));
      case "UndefinedKeyword":
        return Effect.succeed(undefined);
      case "VoidKeyword":
        return Effect.fail(RandomExampleErrorImpl.make(`VoidKeyword`));
      case "NeverKeyword":
        {
          return Effect.fail(RandomExampleErrorImpl.make(`NeverKeyword`));
        }
      case "BigIntKeyword":
        {
          return Effect.map(nextInteger, number => {
            if (constraint) {
              return resolveConstrainedBigint(BigInt(number), constraint);
            }
            return BigInt(number);
          });
        }
      case "SymbolKeyword":
        return Effect.fail(RandomExampleErrorImpl.make(`SymbolKeyword`));
      case "Suspend":
        return go(ast.f(), constraint);
      case "TemplateLiteral":
        {
          const result = (0, _Function.pipe)(ast.spans, Effect.reduce(ast.head, (acc, v) => Effect.map(go(v.type, constraint), x => `${acc}${x}${v.literal}`)));
          return result;
        }
      case "Declaration":
        {
          const description = getDescription(ast);
          const identifier = (0, _Function.pipe)(AST.getAnnotation(AST.IdentifierAnnotationId)(ast), Option.getOrUndefined);
          if (description?.startsWith("Option<")) {
            return (0, _Function.pipe)(randomChoice([() => Effect.succeed(Option.none()), () => Effect.map(go(ast.typeParameters[0], constraint), v => Option.some(v))]), Effect.flatMap(fn => fn()));
          }
          if (identifier === "DateFromSelf") {
            return Effect.succeed(new Date());
          }
          return Effect.fail(RandomExampleErrorImpl.make(`Can't give an example for declaration: ${JSON.stringify(ast)}`));
        }
    }
  };
  return go(schema.ast, undefined).pipe(Effect.provideServiceEffect(IntegerCounter, Ref.make(1)));
};
/** @internal */
exports.randomExample = randomExample;
const createConstraintFromRefinement = /*#__PURE__*/Unify.unify(ast => {
  const typeId = (0, _Function.pipe)(ast, AST.getAnnotation(AST.SchemaIdAnnotationId), Option.getOrUndefined);
  if (typeId === undefined) {
    const astStr = JSON.stringify(ast);
    const message = `Couldn't create an example for ${astStr}. Specify an example.`;
    return Effect.fail(RandomExampleErrorImpl.make(message));
  }
  let from = ast.from;
  while (from._tag === "Refinement") {
    from = from.from;
  }
  let constraint;
  if (from._tag === "NumberKeyword" || from._tag === "BigIntKeyword") {
    constraint = createNumberConstraint(typeId, ast);
  } else if (from._tag === "TupleType") {
    constraint = createTupleConstraint(typeId, ast);
  } else if (from._tag === "Transformation" && from.transformation._tag === "FinalTransformation") {
    return Effect.succeed(TypeConstraint({}));
  }
  if (constraint === undefined) {
    const astStr = JSON.stringify(ast);
    const message = `Couldn't create a constraint for ${astStr} Specify an example.`;
    return Effect.fail(RandomExampleErrorImpl.make(message));
  }
  return Effect.succeed(constraint);
});
/** @internal */
const resolveConstrainedBigint = (number, constraint) => {
  const min = constraint.min && constraint.min + BigInt(constraint.minExclusive ? 1 : 0);
  const max = constraint.max && constraint.max - BigInt(constraint.maxExclusive ? 1 : 0);
  let result = number;
  if (min && number < min) {
    result = min;
  }
  if (max && number > max) {
    result = max;
  }
  return result;
};
/** @internal */
const resolveConstrainedNumber = (number, constraint) => {
  const min = constraint.min && constraint.min + (constraint.minExclusive ? 1 : 0);
  const max = constraint.max && constraint.max - (constraint.maxExclusive ? 1 : 0);
  let result = number;
  if (min && number < min) {
    result = min;
  }
  if (max && number > max) {
    result = max;
  }
  if (constraint.integer && !Number.isInteger(number)) {
    result = Math.ceil(result);
  }
  return result;
};
/** @internal */
const createNumberConstraint = (typeId, ast) => {
  const jsonSchema = ast.annotations[AST.JSONSchemaAnnotationId];
  if (typeId === Schema.IntSchemaId) {
    return TypeConstraint({
      integer: true
    });
  } else if (typeId === Schema.BetweenSchemaId) {
    const [min, max] = [jsonSchema.minimum, jsonSchema.maximum];
    return TypeConstraint({
      min,
      max
    });
  } else if (typeId === Schema.GreaterThanSchemaId) {
    const min = jsonSchema.exclusiveMinimum;
    return TypeConstraint({
      min,
      minExclusive: true
    });
  } else if (typeId === Schema.GreaterThanBigIntSchemaId) {
    const {
      min
    } = ast.annotations[typeId];
    return TypeConstraint({
      min,
      minExclusive: true
    });
  } else if (typeId === Schema.GreaterThanOrEqualToSchemaId) {
    const min = jsonSchema.minimum;
    return TypeConstraint({
      min
    });
  } else if (typeId === Schema.GreaterThanOrEqualToBigIntSchemaId) {
    const {
      min
    } = ast.annotations[typeId];
    return TypeConstraint({
      min
    });
  } else if (typeId === Schema.LessThanSchemaId) {
    const max = jsonSchema.exclusiveMaximum;
    return TypeConstraint({
      max,
      maxExclusive: true
    });
  } else if (typeId === Schema.LessThanBigIntSchemaId) {
    const {
      max
    } = ast.annotations[typeId];
    return TypeConstraint({
      max,
      maxExclusive: true
    });
  } else if (typeId === Schema.LessThanOrEqualToSchemaId) {
    const max = jsonSchema.maximum;
    return TypeConstraint({
      max
    });
  } else if (typeId === Schema.LessThanOrEqualToBigIntSchemaId) {
    const {
      max
    } = ast.annotations[typeId];
    return TypeConstraint({
      max
    });
  }
};
/** @internal */
const createTupleConstraint = (typeId, ast) => {
  const jsonSchema = ast.annotations[AST.JSONSchemaAnnotationId];
  if (typeId === Schema.MinItemsSchemaId) {
    const min = jsonSchema.minItems;
    return TypeConstraint({
      min
    });
  } else if (typeId === Schema.MaxItemsSchemaId) {
    const max = jsonSchema.maxItems;
    return TypeConstraint({
      max
    });
  } else if (typeId === Schema.ItemsCountSchemaId) {
    const min = jsonSchema.minItems;
    const max = jsonSchema.maxItems;
    return TypeConstraint({
      min,
      max
    });
  }
};
/** @internal */
const TypeConstraint = values => ({
  ...values
});
//# sourceMappingURL=example-compiler.js.map