"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.variance = exports.makeRaw = exports.make = exports.getRouter = exports.getEndpoints = exports.empty = exports.concatAll = exports.concat = exports.TypeId = void 0;
var HttpRouter = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("@effect/platform/HttpRouter"));
var Array = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Array"));
var Effect = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Effect"));
var _Function = /*#__PURE__*/require("effect/Function");
var Pipeable = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Pipeable"));
var ApiEndpoint = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiEndpoint.js"));
var HttpError = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../HttpError.js"));
var ServerRequestParser = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("./serverRequestParser.js"));
var ServerResponseEncoder = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("./serverResponseEncoder.js"));
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/** @internal */
const TypeId = exports.TypeId = /*#__PURE__*/Symbol.for("effect-http/Handler/TypeId");
/** @internal */
const variance = exports.variance = {
  /* c8 ignore next */
  _A: _ => _,
  /* c8 ignore next */
  _E: _ => _,
  /* c8 ignore next */
  _R: _ => _
};
/** @internal */
class HandlerImpl {
  endpoints;
  router;
  [TypeId] = variance;
  constructor(endpoints, router) {
    this.endpoints = endpoints;
    this.router = router;
  }
  pipe() {
    return Pipeable.pipeArguments(this, arguments);
  }
}
/** @internal */
const empty = exports.empty = /*#__PURE__*/new HandlerImpl([], HttpRouter.empty); // TODO check variance
/** @internal */
const make = (...args) => {
  if (ApiEndpoint.isApiEndpoint(args[0])) {
    return _make(...args);
  }
  return endpoint => _make(endpoint, ...args);
};
/** @internal */
exports.make = make;
const makeRaw = (...args) => {
  if (ApiEndpoint.isApiEndpoint(args[0])) {
    return _makeRaw(...args);
  }
  return endpoint => _makeRaw(endpoint, args[0]);
};
/** @internal */
exports.makeRaw = makeRaw;
const _makeRaw = (endpoint, handler) => {
  const route = HttpRouter.makeRoute(ApiEndpoint.getMethod(endpoint), ApiEndpoint.getPath(endpoint), handler);
  return new HandlerImpl([endpoint], HttpRouter.fromIterable([route]));
};
/** @internal */
const _make = (endpoint, fn, options) => {
  const responseEncoder = ServerResponseEncoder.create(endpoint);
  const requestParser = ServerRequestParser.create(endpoint, options?.parseOptions);
  const handler = (0, _Function.pipe)(requestParser.parseRequest, Effect.flatMap(input => {
    const {
      security,
      ...restInput
    } = input;
    return fn(restInput, security);
  }), Effect.flatMap(response => responseEncoder.encodeResponse(response)), Effect.catchAll(error => {
    if (HttpError.isHttpError(error)) {
      return HttpError.toResponse(error);
    }
    return Effect.fail(error);
  }));
  return _makeRaw(endpoint, handler);
};
/** @internal */
const getRouter = handler => handler.router;
/** @internal */
exports.getRouter = getRouter;
const getEndpoints = handler => handler.endpoints;
/** @internal */
exports.getEndpoints = getEndpoints;
const concat = exports.concat = /*#__PURE__*/(0, _Function.dual)(2, (self, handler) => new HandlerImpl([...getEndpoints(self), ...getEndpoints(handler)], HttpRouter.concat(getRouter(self), getRouter(handler))));
/** @internal */
const concatAll = (...handlers) => new HandlerImpl(Array.flatten(handlers.map(getEndpoints)), handlers.map(getRouter).reduce(HttpRouter.concat, HttpRouter.empty));
exports.concatAll = concatAll;
//# sourceMappingURL=handler.js.map