"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.variance = exports.merge = exports.mapRouter = exports.make = exports.isRouterBuilder = exports.handler = exports.handleRaw = exports.handle = exports.getRouter = exports.buildPartial = exports.build = exports.addRoute = exports.TypeId = void 0;
var HttpRouter = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("@effect/platform/HttpRouter"));
var Effect = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Effect"));
var _Function = /*#__PURE__*/require("effect/Function");
var Pipeable = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Pipeable"));
var Api = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../Api.js"));
var ApiEndpoint = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiEndpoint.js"));
var Handler = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../Handler.js"));
var HttpError = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../HttpError.js"));
var OpenApi = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../OpenApi.js"));
var SwaggerRouter = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../SwaggerRouter.js"));
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/** @internal */
const TypeId = exports.TypeId = /*#__PURE__*/Symbol.for("effect-http/RouterBuilder/TypeId");
/** @internal */
const variance = exports.variance = {
  /* c8 ignore next */
  _A: _ => _,
  /* c8 ignore next */
  _E: _ => _,
  /* c8 ignore next */
  _R: _ => _
};
/** @internal */
const DEFAULT_OPTIONS = {
  parseOptions: {
    errors: "first",
    onExcessProperty: "ignore"
  },
  enableDocs: true,
  docsPath: "/docs"
};
/** @internal */
class RouterBuilderImpl {
  remainingEndpoints;
  api;
  router;
  options;
  [TypeId] = variance;
  constructor(remainingEndpoints, api, router, options) {
    this.remainingEndpoints = remainingEndpoints;
    this.api = api;
    this.router = router;
    this.options = options;
  }
  pipe() {
    return Pipeable.pipeArguments(this, arguments);
  }
}
/** @internal */
const make = (api, options) => new RouterBuilderImpl(api.groups.flatMap(group => group.endpoints), api, HttpRouter.empty, {
  ...DEFAULT_OPTIONS,
  ...options
});
/** @internal */
exports.make = make;
const handleRaw = (id, handler) => builder => {
  const endpoint = getRemainingEndpoint(builder, id);
  const remainingEndpoints = removeRemainingEndpoint(builder, id);
  const router = builder.router.pipe(HttpRouter.route(ApiEndpoint.getMethod(endpoint))(ApiEndpoint.getPath(endpoint), handler));
  return new RouterBuilderImpl(remainingEndpoints, builder.api, router, builder.options);
};
/** @internal */
exports.handleRaw = handleRaw;
const getRemainingEndpoint = (builder, id) => {
  const endpoint = builder.remainingEndpoints.find(endpoint => ApiEndpoint.getId(endpoint) === id);
  if (endpoint === undefined) {
    throw new Error(`Operation id ${id} not found`);
  }
  return endpoint;
};
/** @internal */
const handle = (...args) => builder => {
  if (args.length === 2) {
    const [id, handler] = args;
    const endpoint = getRemainingEndpoint(builder, id);
    return handle(Handler.make(endpoint, handler))(builder);
  }
  const handler = args[0];
  const handlerEndpointIds = Handler.getEndpoints(handler).map(e => ApiEndpoint.getId(e));
  const remainingEndpoints = builder.remainingEndpoints.filter(e1 => !handlerEndpointIds.includes(ApiEndpoint.getId(e1)));
  const router = HttpRouter.concat(builder.router, Handler.getRouter(handler));
  return new RouterBuilderImpl(remainingEndpoints, builder.api, router, builder.options);
};
/** @internal */
exports.handle = handle;
const handler = (api, id, handler) => Handler.make(Api.getEndpoint(api, id), handler);
/** @internal */
exports.handler = handler;
const mapRouter = fn => builder => new RouterBuilderImpl(builder.remainingEndpoints, builder.api, fn(builder.router), builder.options);
/** @internal */
exports.mapRouter = mapRouter;
const build = builder => buildPartial(builder);
/** @internal */
exports.build = build;
const buildPartial = builder => {
  const swaggerRouter = builder.options.enableDocs ? SwaggerRouter.make(OpenApi.make(builder.api)) : HttpRouter.empty;
  return HttpRouter.mount(builder.router, builder.options.docsPath, swaggerRouter).pipe(Effect.catchTag("RouteNotFound", () => HttpError.toResponse(HttpError.make(404, "Not Found"))));
};
/** @internal */
exports.buildPartial = buildPartial;
const getRouter = builder => builder.router;
/** @internal */
exports.getRouter = getRouter;
const removeRemainingEndpoint = (builder, id) => builder.remainingEndpoints.filter(endpoint => ApiEndpoint.getId(endpoint) !== id);
/** @internal */
const addRoute = (router, route) => HttpRouter.concat(HttpRouter.fromIterable([route]))(router);
/** @internal */
exports.addRoute = addRoute;
const merge = exports.merge = /*#__PURE__*/(0, _Function.dual)(2, (builder1, builder2) => {
  const remainingEndpointIds = builder2.remainingEndpoints.map(e => ApiEndpoint.getId(e));
  if (builder1.api !== builder2.api) {
    throw new Error("Cannot merge router builder for different APIs");
  }
  return new RouterBuilderImpl(builder1.remainingEndpoints.filter(e => remainingEndpointIds.includes(ApiEndpoint.getId(e))), builder1.api, HttpRouter.concat(builder1.router, builder2.router), builder1.options);
});
/** @internal */
const isRouterBuilder = u => typeof u === "object" && u !== null && TypeId in u;
exports.isRouterBuilder = isRouterBuilder;
//# sourceMappingURL=router-builder.js.map