"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.variance = exports.unit = exports.or = exports.never = exports.mapSchema = exports.mapHandler = exports.mapEffect = exports.map = exports.make = exports.handleRequest = exports.getOpenApi = exports.bearer = exports.basic = exports.asSome = exports.as = exports.apiKey = exports.and = exports.TypeId = exports.SecurityImpl = void 0;
var Headers = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("@effect/platform/Headers"));
var HttpServerRequest = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("@effect/platform/HttpServerRequest"));
var Effect = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Effect"));
var Either = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Either"));
var Encoding = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Encoding"));
var _Function = /*#__PURE__*/require("effect/Function");
var Option = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Option"));
var Pipeable = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Pipeable"));
var Schema = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Schema"));
var Unify = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Unify"));
var HttpError = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../HttpError.js"));
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/** @internal */
const TypeId = exports.TypeId = /*#__PURE__*/Symbol.for("effect-http/Security/TypeId");
/** @internal */
const variance = exports.variance = {
  /* c8 ignore next */
  _A: _ => _,
  /* c8 ignore next */
  _E: _ => _,
  /* c8 ignore next */
  _R: _ => _
};
/** @internal */
class SecurityImpl {
  openapi;
  parser;
  [TypeId] = variance;
  constructor(openapi, parser) {
    this.openapi = openapi;
    this.parser = parser;
  }
  pipe() {
    return Pipeable.pipeArguments(this, arguments);
  }
}
/** @internal */
exports.SecurityImpl = SecurityImpl;
const make = (parser, openapi) => new SecurityImpl(openapi ?? {}, parser);
/** @internal */
exports.make = make;
const handleRequest = security => security.parser;
/** @internal */
exports.handleRequest = handleRequest;
const getOpenApi = security => security.openapi;
/** @internal */
exports.getOpenApi = getOpenApi;
const mapHandler = exports.mapHandler = /*#__PURE__*/(0, _Function.dual)(2, (self, f) => make(f(handleRequest(self)), getOpenApi(self)));
/** @internal */
const and = exports.and = /*#__PURE__*/(0, _Function.dual)(2, (self, that) => make(Effect.all([handleRequest(self), handleRequest(that)]), {
  ...getOpenApi(self),
  ...getOpenApi(that)
}));
/** @internal */
const or = exports.or = /*#__PURE__*/(0, _Function.dual)(2, (self, that) => make(Effect.orElse(handleRequest(self), () => handleRequest(that)), {
  ...getOpenApi(self),
  ...getOpenApi(that)
}));
/** @internal */
const mapEffect = exports.mapEffect = /*#__PURE__*/(0, _Function.dual)(2, (self, f) => mapHandler(self, Effect.flatMap(f)));
/** @internal */
const mapSchema = exports.mapSchema = /*#__PURE__*/(0, _Function.dual)(2, (self, schema) => {
  const parse = Schema.decode(schema);
  return make(Effect.flatMap(handleRequest(self), token => (0, _Function.pipe)(parse(token), Effect.mapError(error => HttpError.unauthorized(`Security parsing error, ${error}`)))), getOpenApi(self));
});
/** @internal */
const as = exports.as = /*#__PURE__*/(0, _Function.dual)(2, (self, value) => map(self, () => value));
/** @internal */
const map = exports.map = /*#__PURE__*/(0, _Function.dual)(2, (self, f) => make(Effect.map(handleRequest(self), f), getOpenApi(self)));
const asSome = self => map(self, Option.some);
/** @internal */
exports.asSome = asSome;
const getAuthorizationHeader = /*#__PURE__*/(0, _Function.pipe)(HttpServerRequest.HttpServerRequest, /*#__PURE__*/Effect.flatMap(request => Headers.get(request.headers, "authorization")), /*#__PURE__*/Effect.mapError(() => HttpError.unauthorized("No authorization header")));
/** @internal */
const parseAuthorizationHeader = scheme => Unify.unify(value => {
  const split = value.split(" ");
  if (split.length !== 2) {
    return Effect.fail(HttpError.unauthorized("Invalid authorization header"));
  }
  if (split[0].toLowerCase() !== scheme) {
    return Effect.fail(HttpError.unauthorized(`Expected ${scheme.toUpperCase()} authorization`));
  }
  return Effect.succeed(split[1]);
});
/** @internal */
const bearer = options => {
  const description = options?.description;
  const bearerFormat = options?.bearerFormat;
  const openApi = {
    [options?.name ?? "bearer"]: {
      type: "http",
      ...(description && {
        description
      }),
      ...(bearerFormat && {
        bearerFormat
      }),
      scheme: "bearer"
    }
  };
  return make(Effect.flatMap(getAuthorizationHeader, parseAuthorizationHeader("bearer")), openApi);
};
/** @internal */
exports.bearer = bearer;
const parseBasicAuthCredentials = /*#__PURE__*/Unify.unify(value => (0, _Function.pipe)(Encoding.decodeBase64String(value), Either.mapLeft(() => HttpError.unauthorized("Invalid base64-encoded basic authorization")), Either.flatMap(value => {
  const split = value.split(":");
  if (split.length !== 2) {
    return Either.left(HttpError.unauthorized(`Invalid basic authorization header, expected "user:password".`));
  }
  return Either.right({
    user: split[0],
    pass: split[1]
  });
})));
/** @internal */
const basic = options => {
  const description = options?.description;
  const openApi = {
    [options?.name ?? "basic"]: {
      type: "http",
      ...(description && {
        description
      }),
      scheme: "basic"
    }
  };
  return make((0, _Function.pipe)(getAuthorizationHeader, Effect.flatMap(parseAuthorizationHeader("basic")), Effect.flatMap(parseBasicAuthCredentials)), openApi);
};
/** @internal */
exports.basic = basic;
const unit = exports.unit = /*#__PURE__*/make(Effect.void);
/** @internal */
const never = exports.never = /*#__PURE__*/make(/*#__PURE__*/Effect.fail(/*#__PURE__*/HttpError.unauthorized("No security")));
/** @internal */
const apiKey = options => {
  const schema = Schema.Struct({
    [options.key]: Schema.String
  });
  const parse = (0, _Function.pipe)(Unify.unify(options.in === "query" ? HttpServerRequest.schemaSearchParams(schema) : HttpServerRequest.schemaHeaders(schema)), Effect.map(obj => obj[options.key]), Effect.mapError(() => HttpError.unauthorized(`Expected ${options.key} (${options.in}) api key`)));
  return make(parse, {
    [options.name ?? "apiKey"]: {
      name: options.key,
      type: "apiKey",
      in: options.in,
      ...(options.description && {
        description: options.description
      })
    }
  });
};
exports.apiKey = apiKey;
//# sourceMappingURL=security.js.map