"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.create = void 0;
var HttpRouter = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("@effect/platform/HttpRouter"));
var HttpServerRequest = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("@effect/platform/HttpServerRequest"));
var Effect = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Effect"));
var Schema = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Schema"));
var ApiEndpoint = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiEndpoint.js"));
var ApiRequest = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiRequest.js"));
var ApiSchema = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiSchema.js"));
var HttpError = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../HttpError.js"));
var Security = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../Security.js"));
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/** @internal */
const createError = (location, message) => HttpError.make(400, {
  error: "Request validation error",
  location,
  message
});
/** @internal */
const make = parseRequest => ({
  parseRequest
});
/** @internal */
const create = (endpoint, parseOptions) => make(Effect.all({
  body: parseBody(endpoint, parseOptions),
  query: parseQuery(endpoint, parseOptions),
  path: parsePath(endpoint, parseOptions),
  headers: parseHeaders(endpoint, parseOptions),
  security: parseSecurity(endpoint)
}));
/** @internal */
exports.create = create;
const parseBody = (endpoint, parseOptions) => {
  const schema = ApiRequest.getBodySchema(ApiEndpoint.getRequest(endpoint));
  if (ApiSchema.isIgnored(schema)) {
    return Effect.succeed(undefined);
  }
  const parse = Schema.decodeUnknown(schema);
  if (schema === ApiSchema.FormData) {
    // TODO
    return Effect.succeed(undefined);
  }
  return HttpServerRequest.HttpServerRequest.pipe(Effect.flatMap(request => request.json), Effect.mapError(error => {
    if (error.reason === "Transport") {
      return createError("body", "Unexpect request JSON body error");
    }
    return createError("body", "Invalid JSON");
  }), Effect.flatMap(request => parse(request, parseOptions).pipe(Effect.mapError(error => createError("body", error.message)))));
};
/** @internal */
const parseQuery = (endpoint, parseOptions) => {
  const schema = ApiRequest.getQuerySchema(ApiEndpoint.getRequest(endpoint));
  if (ApiSchema.isIgnored(schema)) {
    return Effect.succeed(undefined);
  }
  return Effect.mapError(HttpServerRequest.schemaSearchParams(schema, parseOptions), error => createError("query", error.message));
};
/** @internal */
const parseHeaders = (endpoint, parseOptions) => {
  const schema = ApiRequest.getHeadersSchema(ApiEndpoint.getRequest(endpoint));
  if (ApiSchema.isIgnored(schema)) {
    return Effect.succeed(undefined);
  }
  const parse = Schema.decodeUnknown(schema);
  return HttpServerRequest.HttpServerRequest.pipe(Effect.flatMap(request => parse(request.headers, parseOptions)), Effect.mapError(error => createError("headers", error.message)));
};
/** @internal */
const parseSecurity = endpoint => {
  const security = ApiEndpoint.getSecurity(endpoint);
  return Security.handleRequest(security);
};
/** @internal */
const parsePath = (endpoint, parseOptions) => {
  const schema = ApiRequest.getPathSchema(ApiEndpoint.getRequest(endpoint));
  if (ApiSchema.isIgnored(schema)) {
    return Effect.succeed(undefined);
  }
  const parse = Schema.decodeUnknown(schema);
  return HttpRouter.RouteContext.pipe(Effect.flatMap(ctx => parse(ctx.params, parseOptions)), Effect.mapError(error => createError("path", error.message)));
};
//# sourceMappingURL=serverRequestParser.js.map