"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.create = void 0;
var HttpBody = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("@effect/platform/HttpBody"));
var HttpServerRequest = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("@effect/platform/HttpServerRequest"));
var HttpServerResponse = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("@effect/platform/HttpServerResponse"));
var Array = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Array"));
var Effect = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Effect"));
var _Function = /*#__PURE__*/require("effect/Function");
var Option = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Option"));
var Schema = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Schema"));
var Unify = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Unify"));
var ApiEndpoint = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiEndpoint.js"));
var ApiResponse = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiResponse.js"));
var ApiSchema = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../ApiSchema.js"));
var HttpError = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("../HttpError.js"));
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/** @internal */
const createErrorResponse = (error, message) => HttpError.make(500, {
  error,
  message
});
/** @internal */
const make = encodeResponse => ({
  encodeResponse
});
/** @internal */
const create = endpoint => {
  const responses = ApiEndpoint.getResponse(endpoint);
  const isFullResponse = ApiEndpoint.isFullResponse(endpoint);
  const statusToSpec = responses.reduce((obj, response) => ({
    ...obj,
    [ApiResponse.getStatus(response)]: response
  }), {});
  return make(inputResponse => (0, _Function.pipe)(Unify.unify(isFullResponse ? Effect.mapError(parseFullResponseInput(inputResponse), () => createErrorResponse("Invalid response", "Server handler returned unexpected response")) : Effect.succeed({
    status: ApiResponse.getStatus(responses[0]),
    body: inputResponse
  })), Effect.zip(HttpServerRequest.HttpServerRequest), Effect.flatMap(([parseInputResponse, request]) => {
    const response = statusToSpec[parseInputResponse.status];
    const setBody = createBodySetter(response);
    const setHeaders = createHeadersSetter(response);
    const representation = representationFromRequest(ApiResponse.getRepresentations(response), request);
    return HttpServerResponse.empty({
      status: parseInputResponse.status
    }).pipe(setBody(parseInputResponse, representation), Effect.flatMap(setHeaders(parseInputResponse)));
  })));
};
/** @internal */
exports.create = create;
const representationFromRequest = (representations, request) => {
  if (representations.length === 0) {
    return representations[0];
  }
  const accept = request.headers["accept"];
  // TODO: this logic needs to be improved a lot!
  return (0, _Function.pipe)(representations, Array.filter(representation => representation.contentType === accept), Array.head, Option.getOrElse(() => representations[0]));
};
/** @internal */
const encodeBody = schema => {
  const encode = Schema.encode(schema);
  return (body, representation) => response => (0, _Function.pipe)(encode(body), Effect.mapError(error => createErrorResponse("Invalid response body", error.message)), Effect.flatMap((0, _Function.flow)(representation.stringify, Effect.mapError(error => createErrorResponse("Invalid response body", error.message)))), Effect.map(body => response.pipe(HttpServerResponse.setBody(HttpBody.text(body, representation.contentType)))));
};
/** @internal */
const createBodySetter = response => {
  const body = ApiResponse.getBodySchema(response);
  const bodySchema = ApiSchema.isIgnored(body) ? undefined : body;
  const encode = bodySchema && encodeBody(bodySchema);
  return (inputResponse, representation) => response => {
    if (encode === undefined && inputResponse.body !== undefined) {
      return Effect.die("Unexpected response body");
    } else if (encode !== undefined && inputResponse.body === undefined) {
      return Effect.die("Response body not provided");
    } else if (encode === undefined) {
      return Effect.succeed(response);
    }
    return (0, _Function.pipe)(response, encode(inputResponse.body, representation));
  };
};
/** @internal */
const createHeadersSetter = schema => {
  const headers = ApiResponse.getHeadersSchema(schema);
  const parseHeaders = ApiSchema.isIgnored(headers) ? undefined : Schema.encode(headers);
  return inputResponse => response => {
    if (parseHeaders === undefined && inputResponse.headers !== undefined) {
      return Effect.die("Unexpected response headers");
    } else if (parseHeaders !== undefined && inputResponse.headers === undefined) {
      return Effect.die("Response headers not provided");
    } else if (parseHeaders === undefined) {
      return Effect.succeed(response);
    }
    return parseHeaders(inputResponse.headers).pipe(Effect.map(headers => response.pipe(HttpServerResponse.setHeaders(headers))), Effect.mapError(error => createErrorResponse("Invalid response headers", error.message)));
  };
};
/** @internal */
const FullResponseInput = /*#__PURE__*/Schema.Struct({
  body: /*#__PURE__*/Schema.optional(Schema.Unknown),
  headers: /*#__PURE__*/Schema.optional(/*#__PURE__*/Schema.Record({
    key: Schema.String,
    value: Schema.Unknown
  })),
  status: Schema.Number
});
/** @internal */
const parseFullResponseInput = /*#__PURE__*/Schema.decodeUnknown(FullResponseInput);
//# sourceMappingURL=serverResponseEncoder.js.map