"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.make = exports.SwaggerFiles = void 0;
var Headers = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("@effect/platform/Headers"));
var HttpRouter = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("@effect/platform/HttpRouter"));
var HttpServerRequest = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("@effect/platform/HttpServerRequest"));
var HttpServerResponse = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("@effect/platform/HttpServerResponse"));
var Array = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Array"));
var Context = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Context"));
var Effect = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("effect/Effect"));
var _Function = /*#__PURE__*/require("effect/Function");
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/**
 * Create a router serving Swagger files.
 *
 * @since 1.0.0
 */

/** @internal */
const createSwaggerInitializer = path => `
window.onload = function() {
  window.ui = SwaggerUIBundle({
    url: "${path}",
    dom_id: '#swagger-ui',
    deepLinking: true,
    presets: [
      SwaggerUIBundle.presets.apis,
      SwaggerUIStandalonePreset
    ],
    plugins: [
      SwaggerUIBundle.plugins.DownloadUrl
    ],
    layout: "StandaloneLayout"
  });
};
`;
/** @internal */
const createIndex = path => `
<!-- HTML for static distribution bundle build -->
<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="UTF-8">
    <title>Swagger UI</title>
    <link rel="stylesheet" type="text/css" href="${path}/swagger-ui.css" />
    <link rel="stylesheet" type="text/css" href="${path}/index.css" />
    <link rel="icon" type="image/png" href="${path}/favicon-32x32.png" sizes="32x32" />
    <link rel="icon" type="image/png" href="${path}/favicon-16x16.png" sizes="16x16" />
  </head>

  <body>
    <div id="swagger-ui"></div>
    <script src="${path}/swagger-ui-bundle.js" charset="UTF-8"> </script>
    <script src="${path}/swagger-ui-standalone-preset.js" charset="UTF-8"> </script>
    <script src="${path}/swagger-initializer.js" charset="UTF-8"> </script>
  </body>
</html>
`;
/** @internal */
const SWAGGER_FILE_NAMES = ["index.css", "swagger-ui.css", "swagger-ui-bundle.js", "swagger-ui-standalone-preset.js", "favicon-32x32.png", "favicon-16x16.png"];
/** @internal */
const SwaggerFiles = exports.SwaggerFiles = /*#__PURE__*/Context.GenericTag("@services/SwaggerFiles");
/** @internal */
const createHeaders = file => {
  if (file.endsWith(".html")) {
    return {
      "content-type": "text/html"
    };
  } else if (file.endsWith(".css")) {
    return {
      "content-type": "text/css"
    };
  } else if (file.endsWith(".js")) {
    return {
      "content-type": "application/javascript"
    };
  } else if (file.endsWith(".png")) {
    return {
      "content-type": "image/png"
    };
  }
  return undefined;
};
/** @internal */
const serverStaticDocsFile = (filename, path) => {
  const headers = createHeaders(filename);
  return HttpRouter.get(path ?? `/${filename}`, Effect.gen(function* (_) {
    const {
      files
    } = yield* _(SwaggerFiles);
    const content = files[filename];
    return HttpServerResponse.raw(content, {
      headers: Headers.fromInput(headers)
    });
  }));
};
/** @internal */
const calculatePrefix = /*#__PURE__*/Effect.gen(function* (_) {
  const request = yield* _(HttpServerRequest.HttpServerRequest);
  const url = request.originalUrl;
  const parts = url.split("/");
  if (!Array.isNonEmptyArray(parts)) {
    return "";
  }
  const last = parts[parts.length - 1];
  if (last.includes(".")) {
    return (0, _Function.pipe)(Array.initNonEmpty(parts), Array.join("/"));
  }
  return url;
});
/** @internal */
const make = spec => {
  let router = SWAGGER_FILE_NAMES.reduce((router, swaggerFileName) => router.pipe(serverStaticDocsFile(swaggerFileName, `/${swaggerFileName}`)), HttpRouter.empty);
  const serveIndex = Effect.gen(function* (_) {
    const prefix = yield* _(calculatePrefix);
    const index = createIndex(prefix);
    return HttpServerResponse.text(index, {
      headers: Headers.fromInput({
        "content-type": "text/html"
      })
    });
  });
  const serveSwaggerInitializer = Effect.gen(function* (_) {
    const prefix = yield* _(calculatePrefix);
    const swaggerInitialiser = createSwaggerInitializer(`${prefix}/openapi.json`);
    return HttpServerResponse.text(swaggerInitialiser, {
      headers: Headers.fromInput({
        "content-type": "application/javascript"
      })
    });
  });
  router = router.pipe(HttpRouter.get(`/`, serveIndex), HttpRouter.get(`/index.html`, serveIndex), HttpRouter.get(`/openapi.json`, Effect.orDie(HttpServerResponse.json(spec))), HttpRouter.get(`/swagger-initializer.js`, serveSwaggerInitializer));
  return router;
};
exports.make = make;
//# sourceMappingURL=swagger-router.js.map