/**
 * Api groups.
 *
 * @since 1.0.0
 */
import type * as Pipeable from "effect/Pipeable";
import type * as Types from "effect/Types";
import type * as ApiEndpoint from "./ApiEndpoint.js";
/**
 * @since 1.0.0
 * @category type id
 */
export declare const TypeId: unique symbol;
/**
 * @since 1.0.0
 * @category type id
 */
export type TypeId = typeof TypeId;
/**
 * @category models
 * @since 1.0.0
 */
export interface Options {
    readonly description?: string;
    readonly externalDocs?: {
        readonly description?: string;
        readonly url: string;
    };
}
/**
 * @category models
 * @since 1.0.0
 */
export interface ApiGroup<A extends ApiEndpoint.ApiEndpoint.Any> extends Pipeable.Pipeable, ApiGroup.Variance<A> {
    readonly name: string;
    readonly endpoints: ReadonlyArray<A>;
    readonly options: Options;
}
/**
 * @category models
 * @since 1.0.0
 */
export declare namespace ApiGroup {
    /**
     * @category models
     * @since 1.0.0
     */
    interface Variance<A extends ApiEndpoint.ApiEndpoint.Any> {
        readonly [TypeId]: {
            readonly _A: Types.Covariant<A>;
        };
    }
    /**
     * Any api group with `Endpoint = Endpoint.Any`.
     *
     * @category models
     * @since 1.0.0
     */
    type Any = ApiGroup<ApiEndpoint.ApiEndpoint.Any>;
    /**
     * Default api group spec.
     *
     * @category models
     * @since 1.0.0
     */
    type Empty = ApiGroup<never>;
    /**
     * @category models
     * @since 1.0.0
     */
    type Context<Endpoint> = [Endpoint] extends [ApiGroup<infer E>] ? ApiEndpoint.ApiEndpoint.Context<E> : never;
}
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const make: (name: string, options?: Partial<Options>) => ApiGroup.Empty;
/**
 * @category modifications
 * @since 1.0.0
 */
export declare const addEndpoint: <E2 extends ApiEndpoint.ApiEndpoint.Any>(endpoint: E2) => <E1 extends ApiEndpoint.ApiEndpoint.Any>(api: ApiGroup<E1>) => ApiGroup<E1 | E2>;
export { 
/**
 * @category modifications
 * @since 1.0.0
 */
addResponse, 
/**
 * @category constructors
 * @since 1.0.0
 */
delete, 
/**
 * @category constructors
 * @since 1.0.0
 */
get, 
/**
 * @category constructors
 * @since 1.0.0
 */
make as endpoint, 
/**
 * @category constructors
 * @since 1.0.0
 */
patch, 
/**
 * @category constructors
 * @since 1.0.0
 */
post, 
/**
 * @category constructors
 * @since 1.0.0
 */
put, 
/**
 * @category modifications
 * @since 1.0.0
 */
setOptions as setEndpointOptions, 
/**
 * @category constructors
 * @since 1.0.0
 */
setRequest, 
/**
 * @category modifications
 * @since 1.0.0
 */
setRequestBody, 
/**
 * @category modifications
 * @since 1.0.0
 */
setRequestHeaders, 
/**
 * @category modifications
 * @since 1.0.0
 */
setRequestPath, 
/**
 * @category modifications
 * @since 1.0.0
 */
setRequestQuery, 
/**
 * @category modifications
 * @since 1.0.0
 */
setResponse, 
/**
 * @category modifications
 * @since 1.0.0
 */
setResponseBody, 
/**
 * @category modifications
 * @since 1.0.0
 */
setResponseHeaders, 
/**
 * @category modifications
 * @since 1.0.0
 */
setResponseRepresentations, 
/**
 * @category modifications
 * @since 1.0.0
 */
setResponseStatus, 
/**
 * @category modifications
 * @since 1.0.0
 */
setSecurity } from "./ApiEndpoint.js";
/**
 * @category modifications
 * @since 1.0.0
 */
export declare const setOptions: (options: Partial<Options>) => <A extends ApiEndpoint.ApiEndpoint.Any>(api: ApiGroup<A>) => ApiGroup<A>;
/**
 * @category refinements
 * @since 1.0.0
 */
export declare const isApiGroup: (u: unknown) => u is ApiGroup.Any;
//# sourceMappingURL=ApiGroup.d.ts.map