/**
 * Models for errors being created on the client side.
 *
 * @since 1.0.0
 */
import type * as Cause from "effect/Cause";
import type * as ParseResult from "effect/ParseResult";
/**
 * @since 1.0.0
 * @category type id
 */
export declare const ClientSideErrorTypeId: unique symbol;
/**
 * @since 1.0.0
 * @category type id
 */
export type ClientSideErrorTypeId = typeof ClientSideErrorTypeId;
/**
 * @since 1.0.0
 * @category type id
 */
export declare const ServerSideErrorTypeId: unique symbol;
/**
 * @since 1.0.0
 * @category type id
 */
export type ServerSideErrorTypeId = typeof ServerSideErrorTypeId;
/**
 * @category models
 * @since 1.0.0
 */
export type ClientError<S extends number = number> = ClientErrorClientSide | ClientErrorServerSide<S>;
/**
 * @category models
 * @since 1.0.0
 */
export interface ClientErrorClientSide extends Cause.YieldableError {
    readonly [ClientSideErrorTypeId]: object;
    readonly _tag: "ClientError";
    readonly message: string;
    readonly error: unknown;
    readonly side: "client";
}
/**
 * @category models
 * @since 1.0.0
 */
export interface ClientErrorServerSide<S extends number = number> extends Cause.YieldableError {
    readonly [ServerSideErrorTypeId]: object;
    readonly _tag: "ClientError";
    readonly message: string;
    readonly error: unknown;
    readonly status: S;
    readonly side: "server";
}
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const makeClientSide: (error: unknown, message?: string) => ClientErrorClientSide;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const makeServerSide: <S extends number>(error: unknown, status: S, message?: string) => ClientErrorServerSide<S>;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const makeClientSideRequestValidation: (location: string) => (error: ParseResult.ParseError) => ClientErrorClientSide;
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const makeClientSideResponseValidation: (location: string) => (error: ParseResult.ParseError) => ClientErrorClientSide;
/**
 * @category refinements
 * @since 1.0.0
 */
export declare const isClientSideError: (u: unknown) => u is ClientErrorClientSide;
/**
 * @category refinements
 * @since 1.0.0
 */
export declare const isServerSideError: (u: unknown) => u is ClientErrorServerSide;
//# sourceMappingURL=ClientError.d.ts.map