/**
 * The `ExampleServer.make` function generates a `RouterBuilder` implementation based
 * on an instance of `Api`. The listening server will perform all the
 * request and response validations similarly to a real implementation.
 *
 * Responses returned from the server are generated randomly using the
 * response `Schema`.
 *
 * @since 1.0.0
 */
import type * as Effect from "effect/Effect";
import type * as Schema from "effect/Schema";
import type * as Api from "./Api.js";
import type * as ApiEndpoint from "./ApiEndpoint.js";
import type * as RouterBuilder from "./RouterBuilder.js";
/**
 * Generate an example `RouterBuilder` implementation.
 *
 * @category utils
 * @since 1.0.0
 */
export declare const make: <A extends Api.Api.Any>(api: A) => RouterBuilder.RouterBuilder<Api.Api.Context<A>, never, never>;
/**
 * Create an example implementation for a single endpoint.
 *
 * @category utils
 * @since 1.0.0
 */
export declare const handle: <A extends ApiEndpoint.ApiEndpoint.Any, Id extends ApiEndpoint.ApiEndpoint.Id<A>>(id: Id) => <R, E>(routerBuilder: RouterBuilder.RouterBuilder<A, E, R>) => RouterBuilder.RouterBuilder<ApiEndpoint.ApiEndpoint.ExcludeById<A, Id>, E, R | ApiEndpoint.ApiEndpoint.Context<ApiEndpoint.ApiEndpoint.ExtractById<A, Id>>>;
/**
 * Create an example implementation for all remaining endpoints.
 *
 * @category utils
 * @since 1.0.0
 */
export declare const handleRemaining: <A extends ApiEndpoint.ApiEndpoint.Any, E, R>(routerBuilder: RouterBuilder.RouterBuilder<A, E, R>) => RouterBuilder.RouterBuilder<never, E, R | ApiEndpoint.ApiEndpoint.Context<A>>;
/**
 * Generate an example value for the given schema.
 *
 * @category utils
 * @since 1.0.0
 */
export declare const makeSchema: <A, Encoded, R>(schema: Schema.Schema<A, Encoded, R>) => Effect.Effect<A>;
//# sourceMappingURL=ExampleServer.d.ts.map