/**
 * `Representation` is a data structure holding information about how to
 * serialize and deserialize a server response for a given conten type.
 *
 * @since 1.0.0
 */
import type * as Cause from "effect/Cause";
import type * as Effect from "effect/Effect";
import type * as Pipeable from "effect/Pipeable";
/**
 * @category type id
 * @since 1.0.0
 */
export declare const TypeId: unique symbol;
/**
 * @category type id
 * @since 1.0.0
 */
export type TypeId = typeof TypeId;
/**
 * @category models
 * @since 1.0.0
 */
export interface Representation extends Pipeable.Pipeable {
    readonly [TypeId]: TypeId;
    readonly stringify: (input: unknown) => Effect.Effect<string, RepresentationError>;
    readonly parse: (input: string) => Effect.Effect<unknown, RepresentationError>;
    contentType: string;
}
/**
 * @category errors
 * @since 1.0.0
 */
export interface RepresentationError extends Cause.YieldableError {
    readonly _tag: "RepresentationError";
    readonly message: string;
}
/**
 * @category constructors
 * @since 1.0.0
 */
export declare const make: (fields: Omit<Representation, TypeId>) => Representation;
/**
 * @category representations
 * @since 1.0.0
 */
export declare const json: Representation;
/**
 * @category representations
 * @since 1.0.0
 */
export declare const plainText: Representation;
//# sourceMappingURL=Representation.d.ts.map