import * as Pipeable from "effect/Pipeable";
import * as Predicate from "effect/Predicate";
import * as ApiSchema from "../ApiSchema.js";
import * as Representation from "../Representation.js";
/** @internal */
export const TypeId = /*#__PURE__*/Symbol.for("effect-http/Api/ResponseTypeId");
/** @internal */
export const variance = {
  /* c8 ignore next */
  _S: _ => _,
  /* c8 ignore next */
  _B: _ => _,
  /* c8 ignore next */
  _H: _ => _,
  /* c8 ignore next */
  _R: _ => _
};
/** @internal */
class ApiResponseImpl {
  status;
  body;
  headers;
  representations;
  [TypeId] = variance;
  constructor(status, body, headers, representations) {
    this.status = status;
    this.body = body;
    this.headers = headers;
    this.representations = representations;
  }
  pipe() {
    return Pipeable.pipeArguments(this, arguments);
  }
}
/** @internal */
const defaultRepresentations = [Representation.json];
/** @internal */
export const defaultResponse = /*#__PURE__*/new ApiResponseImpl(200, ApiSchema.Ignored, ApiSchema.Ignored, defaultRepresentations);
/** @internal */
export const isApiResponse = u => Predicate.hasProperty(u, TypeId) && Predicate.isObject(u[TypeId]);
/** @internal */
export const make = (status, body, headers, representations) => new ApiResponseImpl(status, body ?? ApiSchema.Ignored, headers ?? ApiSchema.Ignored, representations ?? defaultRepresentations);
/** @internal */
export const setStatus = status => response => make(status, getBodySchema(response), getHeadersSchema(response), getRepresentations(response));
/** @internal */
export const setBody = schema => response => make(getStatus(response), schema, getHeadersSchema(response), getRepresentations(response));
/** @internal */
export const setHeaders = schema => response => make(getStatus(response), getBodySchema(response), schema, getRepresentations(response));
/** @internal */
export const setRepresentations = representations => response => make(getStatus(response), getBodySchema(response), getHeadersSchema(response), representations);
/** @internal */
export const getStatus = response => response.status;
/** @internal */
export const getHeadersSchema = response => response.headers;
/** @internal */
export const getBodySchema = response => response.body;
/** @internal */
export const getRepresentations = response => response.representations;
//# sourceMappingURL=api-response.js.map