import * as Array from "effect/Array";
import * as Pipeable from "effect/Pipeable";
import * as ApiEndpoint from "../ApiEndpoint.js";
import * as ApiGroup from "../ApiGroup.js";
import * as api_endpoint from "./api-endpoint.js";
import * as api_group from "./api-group.js";
/** @internal */
export const TypeId = /*#__PURE__*/Symbol.for("effect-http/Api/TypeId");
/** @internal */
export const variance = {
  /* c8 ignore next */
  _A: _ => _
};
/** @internal */
class ApiImpl {
  groups;
  options;
  [TypeId] = variance;
  constructor(groups, options) {
    this.groups = groups;
    this.options = options;
  }
  pipe() {
    return Pipeable.pipeArguments(this, arguments);
  }
}
/** @internal */
const DEFAULT_API_OPTIONS = {
  title: "Api",
  version: "1.0.0"
};
/** @internal */
export const isApi = u => typeof u === "object" && u !== null && TypeId in u;
/** @internal */
export const make = options => new ApiImpl([], {
  ...DEFAULT_API_OPTIONS,
  ...options
});
/** @internal */
export const addEndpoint = endpoint => api => {
  api_endpoint.validateEndpoint(endpoint);
  api.groups.forEach(group => api_group.validateNewEndpoint(group, endpoint));
  const defaultGroup = api.groups.find(group => group.name === "default") ?? ApiGroup.make("default");
  const groupsWithoutDefault = api.groups.filter(group => group.name !== "default");
  const newDefaultGroup = new api_group.ApiGroupImpl("default", [...defaultGroup.endpoints, endpoint], defaultGroup.options);
  return new ApiImpl([...groupsWithoutDefault, newDefaultGroup], api.options);
};
/** @internal */
export const addGroup = group => self => {
  const current = self.groups.flatMap(group => group.endpoints.map(ApiEndpoint.getId));
  const incomming = group.endpoints.map(ApiEndpoint.getId);
  if (Array.intersection(current, incomming).length > 0) {
    throw new Error(`Duplicate endpoint ids found in the group`);
  }
  return new ApiImpl([...self.groups, group], self.options);
};
/** @internal */
export const getEndpoint = (api, id) => {
  const endpoint = api.groups.flatMap(group => group.endpoints).find(endpoint => ApiEndpoint.getId(endpoint) === id);
  if (endpoint === undefined) {
    throw new Error(`Endpoint with id ${id} not found`);
  }
  return endpoint;
};
/** @internal */
export const setOptions = options => self => new ApiImpl(self.groups, {
  ...self.options,
  ...options
});
//# sourceMappingURL=api.js.map