import * as Data from "effect/Data";
import * as Predicate from "effect/Predicate";
/** @internal */
export const ClientSideErrorTypeId = /*#__PURE__*/Symbol.for("effect-http/ClientError/ClientSideErrorTypeId");
/** @internal */
export const ServerSideErrorTypeId = /*#__PURE__*/Symbol.for("effect-http/ClientError/ServerSideErrorTypeId");
/** @internal */
export class ClientErrorServerSideImpl extends /*#__PURE__*/Data.TaggedError("ClientError") {
  [ServerSideErrorTypeId] = {};
}
/** @internal */
export class ClientErrorClientSideImpl extends /*#__PURE__*/Data.TaggedError("ClientError") {
  [ClientSideErrorTypeId] = {};
}
/** @internal */
export const makeClientSide = (error, message) => {
  return new ClientErrorClientSideImpl({
    message: message ?? (typeof error === "string" ? error : JSON.stringify(error)),
    error,
    side: "client"
  });
};
/** @internal */
const getMessage = error => {
  if (typeof error === "string") {
    return error;
  }
  if (error instanceof Error) {
    return error.message;
  }
  if (typeof error === "object" && error !== null && "message" in error && typeof error.message === "string") {
    return error.message;
  }
  return JSON.stringify(error);
};
/** @internal */
export const makeServerSide = (error, status, message) => {
  return new ClientErrorServerSideImpl({
    message: message ?? getMessage(error),
    error,
    status,
    side: "server"
  });
};
/** @internal */
export const makeClientSideRequestValidation = location => error => new ClientErrorClientSideImpl({
  message: `Failed to encode ${location}. ${error.message}`,
  error,
  side: "client"
});
/** @internal */
export const makeClientSideResponseValidation = location => error => new ClientErrorClientSideImpl({
  message: `Failed to validate response ${location}. ${error.message}`,
  error,
  side: "client"
});
/** @internal */
export const isClientSideError = u => Predicate.isTagged(u, "ClientError") && ClientSideErrorTypeId in u;
/** @internal */
export const isServerSideError = u => Predicate.isTagged(u, "ClientError") && ServerSideErrorTypeId in u;
//# sourceMappingURL=client-error.js.map