import * as FetchHttpClient from "@effect/platform/FetchHttpClient";
import * as HttpClient from "@effect/platform/HttpClient";
import * as HttpClientRequest from "@effect/platform/HttpClientRequest";
import * as Context from "effect/Context";
import * as Effect from "effect/Effect";
import * as Encoding from "effect/Encoding";
import { dual, identity, pipe } from "effect/Function";
import * as Layer from "effect/Layer";
import * as Api from "../Api.js";
import * as ApiEndpoint from "../ApiEndpoint.js";
import * as ClientError from "../ClientError.js";
import * as ClientRequestEncoder from "./clientRequestEncoder.js";
import * as ClientResponseParser from "./clientResponseParser.js";
/** @internal */
const defaultHttpClient = /*#__PURE__*/FetchHttpClient.layer.pipe(Layer.build, /*#__PURE__*/Effect.map(/*#__PURE__*/Context.get(HttpClient.HttpClient)), Effect.scoped, Effect.runSync);
/** @internal */
export const endpointClient = (id, api, options) => {
  const endpoint = Api.getEndpoint(api, id);
  const responseParser = ClientResponseParser.create(endpoint);
  const requestEncoder = ClientRequestEncoder.create(endpoint);
  const httpClient = (options.httpClient ?? defaultHttpClient).pipe(options.baseUrl ? HttpClient.mapRequest(HttpClientRequest.prependUrl(options.baseUrl)) : identity);
  return (args, mapper) => pipe(requestEncoder.encodeRequest(args), Effect.map(mapper ?? identity), Effect.flatMap(httpClient.execute), Effect.catchTags({
    RequestError: err => ClientError.makeClientSide(err.cause, err.message),
    ResponseError: err => ClientError.makeServerSide(err.cause, err.response.status, err.message),
    ClientError: err => Effect.fail(err)
  }), Effect.flatMap(responseParser.parseResponse), Effect.annotateLogs("clientOperationId", ApiEndpoint.getId(endpoint)));
};
/** @internal */
export const make = (api, options) => api.groups.flatMap(group => group.endpoints).reduce((client, endpoint) => ({
  ...client,
  [ApiEndpoint.getId(endpoint)]: endpointClient(ApiEndpoint.getId(endpoint), api, options ?? {})
}), {});
/** @internal */
export const setBasicAuth = /*#__PURE__*/dual(3, (request, user, pass) => HttpClientRequest.setHeader(request, "Authorization", `Basic ${Encoding.encodeBase64(`${user}:${pass}`)}`));
/** @internal */
export const setBearer = /*#__PURE__*/dual(2, (request, token) => HttpClientRequest.setHeader(request, "Authorization", `Bearer ${token}`));
/** @internal */
export const setApiKey = /*#__PURE__*/dual(4, (request, key, _in, apiKey) => _in === "query" ? HttpClientRequest.setUrlParam(request, key, apiKey) : HttpClientRequest.setHeader(request, key, apiKey));
//# sourceMappingURL=client.js.map