import * as Array from "effect/Array";
import * as Effect from "effect/Effect";
import { pipe } from "effect/Function";
import * as Unify from "effect/Unify";
import * as Api from "../Api.js";
import * as ApiEndpoint from "../ApiEndpoint.js";
import * as HttpError from "../HttpError.js";
import * as RouterBuilder from "../RouterBuilder.js";
import * as example_compiler from "./example-compiler.js";
import * as utils from "./utils.js";
/** @internal */
export const make = api => handleRemaining(RouterBuilder.make(api));
/** @internal */
export const handle = id => routerBuilder => {
  const endpoint = Api.getEndpoint(routerBuilder.api, id);
  return pipe(routerBuilder, RouterBuilder.handle(id, createExampleHandler(endpoint)));
};
/** @internal */
export const handleRemaining = routerBuilder => pipe(routerBuilder.remainingEndpoints, Array.reduce(routerBuilder, (server, endpoint) => pipe(server, RouterBuilder.handle(ApiEndpoint.getId(endpoint), createExampleHandler(endpoint)))));
/** @internal */
const createExampleHandler = endpoint => {
  const responseSchema = utils.createResponseSchema(endpoint);
  return () => pipe(Unify.unify(responseSchema && example_compiler.randomExample(responseSchema) || Effect.void), Effect.mapError(error => HttpError.internalServerError(`Sorry, I don't have any example response. ${JSON.stringify(error)}`)));
};
export const makeSchema = schema => Effect.orDie(example_compiler.randomExample(schema));
//# sourceMappingURL=example-server.js.map