import * as HttpRouter from "@effect/platform/HttpRouter";
import * as Array from "effect/Array";
import * as Effect from "effect/Effect";
import { dual, pipe } from "effect/Function";
import * as Pipeable from "effect/Pipeable";
import * as ApiEndpoint from "../ApiEndpoint.js";
import * as HttpError from "../HttpError.js";
import * as ServerRequestParser from "./serverRequestParser.js";
import * as ServerResponseEncoder from "./serverResponseEncoder.js";
/** @internal */
export const TypeId = /*#__PURE__*/Symbol.for("effect-http/Handler/TypeId");
/** @internal */
export const variance = {
  /* c8 ignore next */
  _A: _ => _,
  /* c8 ignore next */
  _E: _ => _,
  /* c8 ignore next */
  _R: _ => _
};
/** @internal */
class HandlerImpl {
  endpoints;
  router;
  [TypeId] = variance;
  constructor(endpoints, router) {
    this.endpoints = endpoints;
    this.router = router;
  }
  pipe() {
    return Pipeable.pipeArguments(this, arguments);
  }
}
/** @internal */
export const empty = /*#__PURE__*/new HandlerImpl([], HttpRouter.empty); // TODO check variance
/** @internal */
export const make = (...args) => {
  if (ApiEndpoint.isApiEndpoint(args[0])) {
    return _make(...args);
  }
  return endpoint => _make(endpoint, ...args);
};
/** @internal */
export const makeRaw = (...args) => {
  if (ApiEndpoint.isApiEndpoint(args[0])) {
    return _makeRaw(...args);
  }
  return endpoint => _makeRaw(endpoint, args[0]);
};
/** @internal */
const _makeRaw = (endpoint, handler) => {
  const route = HttpRouter.makeRoute(ApiEndpoint.getMethod(endpoint), ApiEndpoint.getPath(endpoint), handler);
  return new HandlerImpl([endpoint], HttpRouter.fromIterable([route]));
};
/** @internal */
const _make = (endpoint, fn, options) => {
  const responseEncoder = ServerResponseEncoder.create(endpoint);
  const requestParser = ServerRequestParser.create(endpoint, options?.parseOptions);
  const handler = pipe(requestParser.parseRequest, Effect.flatMap(input => {
    const {
      security,
      ...restInput
    } = input;
    return fn(restInput, security);
  }), Effect.flatMap(response => responseEncoder.encodeResponse(response)), Effect.catchAll(error => {
    if (HttpError.isHttpError(error)) {
      return HttpError.toResponse(error);
    }
    return Effect.fail(error);
  }));
  return _makeRaw(endpoint, handler);
};
/** @internal */
export const getRouter = handler => handler.router;
/** @internal */
export const getEndpoints = handler => handler.endpoints;
/** @internal */
export const concat = /*#__PURE__*/dual(2, (self, handler) => new HandlerImpl([...getEndpoints(self), ...getEndpoints(handler)], HttpRouter.concat(getRouter(self), getRouter(handler))));
/** @internal */
export const concatAll = (...handlers) => new HandlerImpl(Array.flatten(handlers.map(getEndpoints)), handlers.map(getRouter).reduce(HttpRouter.concat, HttpRouter.empty));
//# sourceMappingURL=handler.js.map