import * as Cookies from "@effect/platform/Cookies";
import * as Headers from "@effect/platform/Headers";
import * as HttpBody from "@effect/platform/HttpBody";
import * as HttpServerResponse from "@effect/platform/HttpServerResponse";
import * as Data from "effect/Data";
import { absurd, identity, pipe } from "effect/Function";
import * as Pipeable from "effect/Pipeable";
import * as Stream from "effect/Stream";
/** @internal */
export const TypeId = /*#__PURE__*/Symbol.for("effect-http/HttpError/TypeId");
/** @internal */
export const variance = {};
/** @internal */
const unsafeIsStream = u => typeof u === "object" && u !== null && Stream.StreamTypeId in u;
/** @internal */
export class HttpErrorImpl extends /*#__PURE__*/Data.TaggedError("HttpError") {
  [TypeId] = variance;
  static unsafeFromUnknown(status, body, options) {
    let content = undefined;
    if (body === undefined) {
      content = HttpBody.empty;
    } else if (body instanceof Uint8Array) {
      content = HttpBody.uint8Array(body);
    } else if (typeof body === "string") {
      content = HttpBody.text(body);
    } else if (unsafeIsStream(body)) {
      content = HttpBody.stream(body);
    } else {
      content = HttpBody.unsafeJson(body);
    }
    return new HttpErrorImpl({
      status,
      content,
      headers: options?.headers ?? Headers.empty,
      cookies: options?.cookies ?? Cookies.empty
    });
  }
  pipe() {
    return Pipeable.pipeArguments(this, arguments);
  }
}
/** @internal */
export const toResponse = error => {
  const options = {
    status: error.status,
    contentType: error.content.contentType,
    headers: pipe(error.headers, error.content.contentLength ? Headers.set("Content-Length", error.content.contentLength.toString()) : identity),
    cookies: error.cookies
  };
  const content = error.content;
  if (content._tag === "Empty") {
    return HttpServerResponse.empty(options);
  } else if (content._tag === "Uint8Array") {
    return HttpServerResponse.uint8Array(content.body, options);
  } else if (content._tag === "Raw") {
    return HttpServerResponse.raw(content.body, options);
  } else if (content._tag === "Stream") {
    return HttpServerResponse.stream(content.stream, options);
  } else if (content._tag === "FormData") {
    return HttpServerResponse.formData(content.formData, options);
  }
  return absurd(content);
};
/** @internal */
export const make = (status, body, options) => HttpErrorImpl.unsafeFromUnknown(status, body, options);
export const makeStatus = status => (body, options) => HttpErrorImpl.unsafeFromUnknown(status, body, options);
/** @internal */
export const isHttpError = error => typeof error === "object" && error !== null && TypeId in error;
//# sourceMappingURL=http-error.js.map