import * as Effect from "effect/Effect";
import { identity, pipe } from "effect/Function";
import * as ApiEndpoint from "../ApiEndpoint.js";
import * as ClientRequestEncoder from "./clientRequestEncoder.js";
import * as example_compiler from "./example-compiler.js";
import * as utils from "./utils.js";
/** @internal */
export const make = (api, option) => api.groups.flatMap(group => group.endpoints).reduce((client, endpoint) => {
  const requestEncoder = ClientRequestEncoder.create(endpoint);
  const responseSchema = utils.createResponseSchema(endpoint);
  const customResponses = option?.responses;
  const customResponse = customResponses && customResponses[ApiEndpoint.getId(endpoint)];
  const fn = (args, mapRequest) => {
    return pipe(requestEncoder.encodeRequest(args), Effect.map(mapRequest ?? identity), Effect.flatMap(() => {
      if (customResponse !== undefined) {
        return Effect.succeed(customResponse);
      } else if (responseSchema === undefined) {
        return Effect.void;
      }
      return example_compiler.randomExample(responseSchema);
    }));
  };
  return {
    ...client,
    [ApiEndpoint.getId(endpoint)]: fn
  };
}, {});
//# sourceMappingURL=mock-client.js.map