import * as Data from "effect/Data";
import * as Effect from "effect/Effect";
import * as Pipeable from "effect/Pipeable";
/** @internal */
export const TypeId = /*#__PURE__*/Symbol.for("effect-http/Representation/Representation");
/** @internal */
class RepresentationErrorImpl extends /*#__PURE__*/Data.TaggedError("RepresentationError") {}
/** @internal */
const representationProto = {
  [TypeId]: TypeId,
  pipe() {
    return Pipeable.pipeArguments(this, arguments);
  }
};
/** @internal */
export const make = fields => {
  const representation = Object.create(representationProto);
  return Object.assign(representation, fields);
};
/** @internal */
export const json = /*#__PURE__*/make({
  stringify: input => Effect.try({
    try: () => JSON.stringify(input),
    catch: error => new RepresentationErrorImpl({
      message: `JSON parsing failed with ${error}`
    })
  }),
  parse: input => Effect.try({
    try: () => JSON.parse(input),
    catch: error => new RepresentationErrorImpl({
      message: `JSON stringify failed with ${error}`
    })
  }),
  contentType: "application/json"
});
/** @internal */
export const plainText = /*#__PURE__*/make({
  stringify: input => {
    if (typeof input === "string") {
      return Effect.succeed(input);
    } else if (typeof input === "number") {
      return Effect.succeed(String(input));
    }
    return json.stringify(input);
  },
  parse: Effect.succeed,
  contentType: "text/plain"
});
//# sourceMappingURL=representation.js.map