import * as HttpRouter from "@effect/platform/HttpRouter";
import * as Effect from "effect/Effect";
import { dual } from "effect/Function";
import * as Pipeable from "effect/Pipeable";
import * as Api from "../Api.js";
import * as ApiEndpoint from "../ApiEndpoint.js";
import * as Handler from "../Handler.js";
import * as HttpError from "../HttpError.js";
import * as OpenApi from "../OpenApi.js";
import * as SwaggerRouter from "../SwaggerRouter.js";
/** @internal */
export const TypeId = /*#__PURE__*/Symbol.for("effect-http/RouterBuilder/TypeId");
/** @internal */
export const variance = {
  /* c8 ignore next */
  _A: _ => _,
  /* c8 ignore next */
  _E: _ => _,
  /* c8 ignore next */
  _R: _ => _
};
/** @internal */
const DEFAULT_OPTIONS = {
  parseOptions: {
    errors: "first",
    onExcessProperty: "ignore"
  },
  enableDocs: true,
  docsPath: "/docs"
};
/** @internal */
class RouterBuilderImpl {
  remainingEndpoints;
  api;
  router;
  options;
  [TypeId] = variance;
  constructor(remainingEndpoints, api, router, options) {
    this.remainingEndpoints = remainingEndpoints;
    this.api = api;
    this.router = router;
    this.options = options;
  }
  pipe() {
    return Pipeable.pipeArguments(this, arguments);
  }
}
/** @internal */
export const make = (api, options) => new RouterBuilderImpl(api.groups.flatMap(group => group.endpoints), api, HttpRouter.empty, {
  ...DEFAULT_OPTIONS,
  ...options
});
/** @internal */
export const handleRaw = (id, handler) => builder => {
  const endpoint = getRemainingEndpoint(builder, id);
  const remainingEndpoints = removeRemainingEndpoint(builder, id);
  const router = builder.router.pipe(HttpRouter.route(ApiEndpoint.getMethod(endpoint))(ApiEndpoint.getPath(endpoint), handler));
  return new RouterBuilderImpl(remainingEndpoints, builder.api, router, builder.options);
};
/** @internal */
const getRemainingEndpoint = (builder, id) => {
  const endpoint = builder.remainingEndpoints.find(endpoint => ApiEndpoint.getId(endpoint) === id);
  if (endpoint === undefined) {
    throw new Error(`Operation id ${id} not found`);
  }
  return endpoint;
};
/** @internal */
export const handle = (...args) => builder => {
  if (args.length === 2) {
    const [id, handler] = args;
    const endpoint = getRemainingEndpoint(builder, id);
    return handle(Handler.make(endpoint, handler))(builder);
  }
  const handler = args[0];
  const handlerEndpointIds = Handler.getEndpoints(handler).map(e => ApiEndpoint.getId(e));
  const remainingEndpoints = builder.remainingEndpoints.filter(e1 => !handlerEndpointIds.includes(ApiEndpoint.getId(e1)));
  const router = HttpRouter.concat(builder.router, Handler.getRouter(handler));
  return new RouterBuilderImpl(remainingEndpoints, builder.api, router, builder.options);
};
/** @internal */
export const handler = (api, id, handler) => Handler.make(Api.getEndpoint(api, id), handler);
/** @internal */
export const mapRouter = fn => builder => new RouterBuilderImpl(builder.remainingEndpoints, builder.api, fn(builder.router), builder.options);
/** @internal */
export const build = builder => buildPartial(builder);
/** @internal */
export const buildPartial = builder => {
  const swaggerRouter = builder.options.enableDocs ? SwaggerRouter.make(OpenApi.make(builder.api)) : HttpRouter.empty;
  return HttpRouter.mount(builder.router, builder.options.docsPath, swaggerRouter).pipe(Effect.catchTag("RouteNotFound", () => HttpError.toResponse(HttpError.make(404, "Not Found"))));
};
/** @internal */
export const getRouter = builder => builder.router;
/** @internal */
const removeRemainingEndpoint = (builder, id) => builder.remainingEndpoints.filter(endpoint => ApiEndpoint.getId(endpoint) !== id);
/** @internal */
export const addRoute = (router, route) => HttpRouter.concat(HttpRouter.fromIterable([route]))(router);
/** @internal */
export const merge = /*#__PURE__*/dual(2, (builder1, builder2) => {
  const remainingEndpointIds = builder2.remainingEndpoints.map(e => ApiEndpoint.getId(e));
  if (builder1.api !== builder2.api) {
    throw new Error("Cannot merge router builder for different APIs");
  }
  return new RouterBuilderImpl(builder1.remainingEndpoints.filter(e => remainingEndpointIds.includes(ApiEndpoint.getId(e))), builder1.api, HttpRouter.concat(builder1.router, builder2.router), builder1.options);
});
/** @internal */
export const isRouterBuilder = u => typeof u === "object" && u !== null && TypeId in u;
//# sourceMappingURL=router-builder.js.map