import * as Schema from "effect/Schema";
import * as ApiEndpoint from "../ApiEndpoint.js";
import * as ApiResponse from "../ApiResponse.js";
import * as ApiSchema from "../ApiSchema.js";
/** @internal */
export const getSchema = (input, defaultSchema = Schema.Unknown) => ApiSchema.isIgnored(input) ? defaultSchema : input;
/** @internal */
export const createResponseSchema = endpoint => {
  const response = ApiEndpoint.getResponse(endpoint);
  const isFullResponse = ApiEndpoint.isFullResponse(endpoint);
  if (!isFullResponse && ApiSchema.isIgnored(ApiResponse.getBodySchema(response[0]))) {
    return undefined;
  }
  if (!isFullResponse) {
    return getSchema(ApiResponse.getBodySchema(response[0]));
  }
  return Schema.Union(...response.map(response => Schema.Struct({
    status: Schema.Literal(ApiResponse.getStatus(response)),
    body: getSchema(ApiResponse.getBodySchema(response)),
    headers: getSchema(ApiResponse.getHeadersSchema(response), Schema.Record({
      key: Schema.String,
      value: Schema.String
    }))
  })));
};
//# sourceMappingURL=utils.js.map